@@ -1,4 +1,4 @@
-from __future__ import with_statement
+from __future__ import absolute_import, division, print_function, unicode_literals
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -6,6 +6,7 @@ import os, sys, zipfile, importlib
 
 from calibre.constants import numeric_version, iswindows, isosx
 from calibre.ptempfile import PersistentTemporaryFile
+from polyglot.builtins import unicode_type
 
 platform = 'linux'
 if iswindows:
@@ -64,7 +65,7 @@ class Plugin(object):  # {{{
     #: When more than one plugin exists for a filetype,
     #: the plugins are run in order of decreasing priority.
     #: Plugins with higher priority will be run first.
-    #: The highest possible priority is ``sys.maxint``.
+    #: The highest possible priority is ``sys.maxsize``.
     #: Default priority is 1.
     priority = 1
 
@@ -194,7 +195,7 @@ class Plugin(object):  # {{{
             config_dialog.exec_()
 
             if config_dialog.result() == QDialog.Accepted:
-                sc = unicode(sc.text()).strip()
+                sc = unicode_type(sc.text()).strip()
                 customize_plugin(self, sc)
 
         geom = bytearray(config_dialog.saveGeometry())
@@ -210,7 +211,7 @@ class Plugin(object):  # {{{
         For example to load an image::
 
             pixmap = QPixmap()
-            pixmap.loadFromData(self.load_resources(['images/icon.png']).itervalues().next())
+            pixmap.loadFromData(self.load_resources(['images/icon.png'])['images/icon.png'])
             icon = QIcon(pixmap)
 
         :param names: List of paths to resources in the ZIP file using / as separator
@@ -246,7 +247,7 @@ class Plugin(object):  # {{{
         :param gui: If True return HTML help, otherwise return plain text help.
 
         '''
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def temporary_file(self, suffix):
         '''
@@ -276,8 +277,8 @@ class Plugin(object):  # {{{
         if self.plugin_path is not None:
             from calibre.utils.zipfile import ZipFile
             zf = ZipFile(self.plugin_path)
-            extensions = set([x.rpartition('.')[-1].lower() for x in
-                zf.namelist()])
+            extensions = {x.rpartition('.')[-1].lower() for x in
+                zf.namelist()}
             zip_safe = True
             for ext in ('pyd', 'so', 'dll', 'dylib'):
                 if ext in extensions:
@@ -404,7 +405,7 @@ class MetadataReaderPlugin(Plugin):  # {{{
     '''
     #: Set of file types for which this plugin should be run.
     #: For example: ``set(['lit', 'mobi', 'prc'])``
-    file_types     = set([])
+    file_types     = set()
 
     supported_platforms = ['windows', 'osx', 'linux']
     version = numeric_version
@@ -436,7 +437,7 @@ class MetadataWriterPlugin(Plugin):  # {{{
     '''
     #: Set of file types for which this plugin should be run.
     #: For example: ``set(['lit', 'mobi', 'prc'])``
-    file_types     = set([])
+    file_types     = set()
 
     supported_platforms = ['windows', 'osx', 'linux']
     version = numeric_version
@@ -472,7 +473,7 @@ class CatalogPlugin(Plugin):  # {{{
 
     #: Output file type for which this plugin should be run.
     #: For example: 'epub' or 'xml'
-    file_types = set([])
+    file_types = set()
 
     type = _('Catalog generator')
 
@@ -506,11 +507,10 @@ class CatalogPlugin(Plugin):  # {{{
 
     def get_output_fields(self, db, opts):
         # Return a list of requested fields
-        all_std_fields = set(
-                          ['author_sort','authors','comments','cover','formats',
+        all_std_fields = {'author_sort','authors','comments','cover','formats',
                            'id','isbn','library_name','ondevice','pubdate','publisher',
                            'rating','series_index','series','size','tags','timestamp',
-                           'title_sort','title','uuid','languages','identifiers'])
+                           'title_sort','title','uuid','languages','identifiers'}
         all_custom_fields = set(db.custom_field_keys())
         for field in list(all_custom_fields):
             fm = db.field_metadata[field]
@@ -563,7 +563,7 @@ class CatalogPlugin(Plugin):  # {{{
                 try:
                     resources.extract(file, self.resources_path)
                 except:
-                    print " customize:__init__.initialize(): %s not found in %s" % (file, os.path.basename(self.plugin_path))
+                    print(" customize:__init__.initialize(): %s not found in %s" % (file, os.path.basename(self.plugin_path)))
                     continue
             resources.close()
 
@@ -743,7 +743,7 @@ class ViewerPlugin(Plugin):  # {{{
             def load_fonts():
                 from PyQt5.Qt import QFontDatabase
                 font_data = get_resources(['myfont1.ttf', 'myfont2.ttf'])
-                for raw in font_data.itervalues():
+                for raw in font_data.values():
                     QFontDatabase.addApplicationFontFromData(raw)
         '''
         pass
