@@ -37,8 +37,8 @@ def _config():
     c.add_opt('plugins', default={}, help=_('Installed plugins'))
     c.add_opt('filetype_mapping', default={}, help=_('Mapping for filetype plugins'))
     c.add_opt('plugin_customization', default={}, help=_('Local plugin customization'))
-    c.add_opt('disabled_plugins', default=set([]), help=_('Disabled plugins'))
-    c.add_opt('enabled_plugins', default=set([]), help=_('Enabled plugins'))
+    c.add_opt('disabled_plugins', default=set(), help=_('Disabled plugins'))
+    c.add_opt('enabled_plugins', default=set(), help=_('Enabled plugins'))
 
     return ConfigProxy(c)
 
@@ -307,14 +307,14 @@ def available_store_plugins():
 
 
 def stores():
-    stores = set([])
+    stores = set()
     for plugin in store_plugins():
         stores.add(plugin.name)
     return stores
 
 
 def available_stores():
-    stores = set([])
+    stores = set()
     for plugin in available_store_plugins():
         stores.add(plugin.name)
     return stores
@@ -575,7 +575,7 @@ def catalog_plugins():
 
 
 def available_catalog_formats():
-    formats = set([])
+    formats = set()
     for plugin in catalog_plugins():
         if not is_disabled(plugin):
             for format in plugin.file_types:
