@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,7 +8,7 @@ __docformat__ = 'restructuredtext en'
 
 import copy
 from functools import partial
-from polyglot.builtins import iteritems, unicode_type, map
+from polyglot.builtins import iteritems, unicode_type, map, native_string_type
 
 from calibre.constants import ispy3
 from calibre.ebooks.metadata import author_to_author_sort
@@ -59,7 +58,7 @@ class Tag(object):
             return self.string_representation
 
     def __repr__(self):
-        return str(self)
+        return native_string_type(self)
 
     __calibre_serializable__ = True
 
