@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -12,11 +11,12 @@ from threading import RLock
 from collections import namedtuple
 from functools import partial
 
-from calibre import prints, as_unicode
-from calibre.constants import plugins, islinux, isosx
+from calibre import prints, as_unicode, force_unicode
+from calibre.constants import plugins, islinux, isosx, ispy3
 from calibre.ptempfile import SpooledTemporaryFile
 from calibre.devices.errors import OpenFailed, DeviceError, BlacklistedDevice, OpenActionNeeded
 from calibre.devices.mtp.base import MTPDeviceBase, synchronous, debug
+from polyglot.builtins import unicode_type
 
 MTPDevice = namedtuple('MTPDevice', 'busnum devnum vendor_id product_id '
         'bcd serial manufacturer product')
@@ -34,8 +34,7 @@ APPLE = 0x05ac
 
 class MTP_DEVICE(MTPDeviceBase):
 
-    # libusb(x) does not work on OS X. So no MTP support for OS X
-    supported_platforms = ['linux', 'osx']
+    supported_platforms = ['freebsd', 'linux', 'osx']
 
     def __init__(self, *args, **kwargs):
         MTPDeviceBase.__init__(self, *args, **kwargs)
@@ -167,8 +166,12 @@ class MTP_DEVICE(MTPDeviceBase):
     @synchronous
     def create_device(self, connected_device):
         d = connected_device
+        man, prod = d.manufacturer, d.product
+        if ispy3:
+            man = force_unicode(man, 'utf-8') if isinstance(man, bytes) else man
+            prod = force_unicode(prod, 'utf-8') if isinstance(prod, bytes) else prod
         return self.libmtp.Device(d.busnum, d.devnum, d.vendor_id,
-                d.product_id, d.manufacturer, d.product, d.serial)
+                d.product_id, man, prod, d.serial)
 
     @synchronous
     def eject(self):
@@ -192,8 +195,8 @@ class MTP_DEVICE(MTPDeviceBase):
         p = plugins['libmtp']
         self.libmtp = p[0]
         if self.libmtp is None:
-            print ('Failed to load libmtp, MTP device detection disabled')
-            print (p[1])
+            print('Failed to load libmtp, MTP device detection disabled')
+            print(p[1])
         else:
             self.known_devices = frozenset(self.libmtp.known_devices())
 
@@ -223,7 +226,7 @@ class MTP_DEVICE(MTPDeviceBase):
         try:
             storage = sorted(self.dev.storage_info, key=operator.itemgetter('id'))
         except self.libmtp.MTPError as e:
-            if "The device has no storage information." in str(e):
+            if "The device has no storage information." in unicode_type(e):
                 # This happens on newer Android devices while waiting for
                 # the user to allow access. Apparently what happens is
                 # that when the user clicks allow, the device disconnects
@@ -318,7 +321,7 @@ class MTP_DEVICE(MTPDeviceBase):
                     storage.append({'id':sid, 'size':capacity,
                         'is_folder':True, 'name':name, 'can_delete':False,
                         'is_system':True})
-                    self._currently_getting_sid = unicode(sid)
+                    self._currently_getting_sid = unicode_type(sid)
                     items, errs = self.dev.get_filesystem(sid,
                             partial(self._filesystem_callback, {}))
                     all_items.extend(items), all_errs.extend(errs)
@@ -370,7 +373,7 @@ class MTP_DEVICE(MTPDeviceBase):
         e = parent.folder_named(name)
         if e is not None:
             return e
-        ename = name.encode('utf-8') if isinstance(name, unicode) else name
+        ename = name.encode('utf-8') if isinstance(name, unicode_type) else name
         sid, pid = parent.storage_id, parent.object_id
         if pid == sid:
             pid = 0
@@ -393,7 +396,7 @@ class MTP_DEVICE(MTPDeviceBase):
                 raise ValueError('Cannot upload file %s, it already exists'%(
                     e.full_path,))
             self.delete_file_or_folder(e)
-        ename = name.encode('utf-8') if isinstance(name, unicode) else name
+        ename = name.encode('utf-8') if isinstance(name, unicode_type) else name
         sid, pid = parent.storage_id, parent.object_id
         if pid == sid:
             pid = 0xFFFFFFFF
