@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,15 +8,16 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 from collections import defaultdict
 from functools import partial
 
-from cssutils.css import CSSRule, CSSStyleDeclaration
+from css_parser.css import CSSRule, CSSStyleDeclaration
 from css_selectors import parse, SelectorSyntaxError
 
 from calibre import force_unicode
-from calibre.ebooks.oeb.base import OEB_STYLES, OEB_DOCS
+from calibre.ebooks.oeb.base import OEB_STYLES, OEB_DOCS, XHTML, css_text
 from calibre.ebooks.oeb.normalize_css import normalize_filter_css, normalizers
-from calibre.ebooks.oeb.polish.pretty import pretty_script_or_style
+from calibre.ebooks.oeb.polish.pretty import pretty_script_or_style, pretty_xml_tree, serialize
 from calibre.utils.icu import numeric_sort_key
 from css_selectors import Select, SelectorError
+from polyglot.builtins import iteritems, itervalues, unicode_type, filter
 
 
 def filter_used_rules(rules, log, select):
@@ -63,7 +63,7 @@ def merge_identical_selectors(sheet):
     for rule in sheet.cssRules.rulesOfType(CSSRule.STYLE_RULE):
         selector_map[rule.selectorText].append(rule)
     remove = []
-    for rule_group in selector_map.itervalues():
+    for rule_group in itervalues(selector_map):
         if len(rule_group) > 1:
             for i in range(1, len(rule_group)):
                 merge_declarations(rule_group[0].style, rule_group[i].style)
@@ -88,23 +88,23 @@ def remove_unused_css(container, report=None, remove_unused_classes=False, merge
             return container.parsed(name)
         except TypeError:
             pass
-    sheets = {name:safe_parse(name) for name, mt in container.mime_map.iteritems() if mt in OEB_STYLES}
-    sheets = {k:v for k, v in sheets.iteritems() if v is not None}
+    sheets = {name:safe_parse(name) for name, mt in iteritems(container.mime_map) if mt in OEB_STYLES}
+    sheets = {k:v for k, v in iteritems(sheets) if v is not None}
     num_merged = 0
     if merge_rules:
-        for name, sheet in sheets.iteritems():
+        for name, sheet in iteritems(sheets):
             num = merge_identical_selectors(sheet)
             if num:
                 container.dirty(name)
                 num_merged += num
     import_map = {name:get_imported_sheets(name, container, sheets) for name in sheets}
     if remove_unused_classes:
-        class_map = {name:{icu_lower(x) for x in classes_in_rule_list(sheet.cssRules)} for name, sheet in sheets.iteritems()}
-    style_rules = {name:tuple(sheet.cssRules.rulesOfType(CSSRule.STYLE_RULE)) for name, sheet in sheets.iteritems()}
+        class_map = {name:{icu_lower(x) for x in classes_in_rule_list(sheet.cssRules)} for name, sheet in iteritems(sheets)}
+    style_rules = {name:tuple(sheet.cssRules.rulesOfType(CSSRule.STYLE_RULE)) for name, sheet in iteritems(sheets)}
 
     num_of_removed_rules = num_of_removed_classes = 0
 
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt not in OEB_DOCS:
             continue
         root = container.parsed(name)
@@ -161,7 +161,7 @@ def remove_unused_css(container, report=None, remove_unused_classes=False, merge
                     num_of_removed_classes += len(original_classes) - len(classes)
                     container.dirty(name)
 
-    for name, sheet in sheets.iteritems():
+    for name, sheet in iteritems(sheets):
         unused_rules = style_rules[name]
         if unused_rules:
             num_of_removed_rules += len(unused_rules)
@@ -208,7 +208,7 @@ def filter_declaration(style, properties=()):
 
 
 def filter_sheet(sheet, properties=()):
-    from cssutils.css import CSSRule
+    from css_parser.css import CSSRule
     changed = False
     remove = []
     for rule in sheet.cssRules.rulesOfType(CSSRule.STYLE_RULE):
@@ -225,7 +225,7 @@ def transform_css(container, transform_sheet=None, transform_style=None, names=(
     if not names:
         types = OEB_STYLES | OEB_DOCS
         names = []
-        for name, mt in container.mime_map.iteritems():
+        for name, mt in iteritems(container.mime_map):
             if mt in types:
                 names.append(name)
 
@@ -324,14 +324,13 @@ def remove_property_value(prop, predicate):
     values of the property would be removed, the property is removed from its
     parent instead. Note that this means the property must have a parent (a
     CSSStyleDeclaration). '''
-    removed_vals = []
-    removed_vals = filter(predicate, prop.propertyValue)
+    removed_vals = list(filter(predicate, prop.propertyValue))
     if len(removed_vals) == len(prop.propertyValue):
         prop.parent.removeProperty(prop.name)
     else:
-        x = prop.propertyValue.cssText
+        x = css_text(prop.propertyValue)
         for v in removed_vals:
-            x = x.replace(v.cssText, '').strip()
+            x = x.replace(css_text(v), '').strip()
         prop.propertyValue.cssText = x
     return bool(removed_vals)
 
@@ -343,10 +342,10 @@ def sort_sheet(container, sheet_or_text):
     ''' Sort the rules in a stylesheet. Note that in the general case this can
     change the effective styles, but for most common sheets, it should be safe.
     '''
-    sheet = container.parse_css(sheet_or_text) if isinstance(sheet_or_text, unicode) else sheet_or_text
+    sheet = container.parse_css(sheet_or_text) if isinstance(sheet_or_text, unicode_type) else sheet_or_text
 
     def text_sort_key(x):
-        return numeric_sort_key(unicode(x or ''))
+        return numeric_sort_key(unicode_type(x or ''))
 
     def selector_sort_key(x):
         return (x.specificity, text_sort_key(x.selectorText))
@@ -369,3 +368,19 @@ def sort_sheet(container, sheet_or_text):
         return primary, secondary, tertiary
     sheet.cssRules.sort(key=rule_sort_key)
     return sheet
+
+
+def add_stylesheet_links(container, name, text):
+    root = container.parse_xhtml(text, name)
+    head = root.xpath('//*[local-name() = "head"]')
+    if not head:
+        return
+    head = head[0]
+    sheets = tuple(container.manifest_items_of_type(lambda mt: mt in OEB_STYLES))
+    if not sheets:
+        return
+    for sname in sheets:
+        link = head.makeelement(XHTML('link'), type='text/css', rel='stylesheet', href=container.name_to_href(sname, name))
+        head.append(link)
+    pretty_xml_tree(head)
+    return serialize(root, 'text/html')
