@@ -1,7 +1,7 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
 '''
 CSS case-mangling transform.
 '''
-from __future__ import with_statement
 
 __license__   = 'GPL v3'
 __copyright__ = '2008, Marshall T. Vandegrift <llasram@gmail.com>'
