@@ -1,32 +1,52 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
-""" The GUI """
-import os, sys, Queue, threading, glob, signal
-from contextlib import contextmanager
-from threading import RLock, Lock
-from PyQt5.QtWidgets import QStyle  # Gives a nicer error message than import from Qt
-from PyQt5.Qt import (
-    QFileInfo, QObject, QBuffer, Qt, QByteArray, QTranslator, QSocketNotifier,
-    QCoreApplication, QThread, QEvent, QTimer, pyqtSignal, QDateTime, QFontMetrics,
-    QDesktopServices, QFileDialog, QFileIconProvider, QSettings, QIcon, QStringListModel,
-    QApplication, QDialog, QUrl, QFont, QFontDatabase, QLocale, QFontInfo, QT_VERSION)
 
-from calibre import prints, as_unicode
-from calibre.constants import (islinux, iswindows, isbsd, isfrozen, isosx, is_running_from_develop,
-        plugins, config_dir, filesystem_encoding, isxp, DEBUG, __version__, __appname__ as APP_UID)
-from calibre.ptempfile import base_dir
-from calibre.gui2.linux_file_dialogs import check_for_linux_native_dialogs, linux_native_dialog
-from calibre.gui2.qt_file_dialogs import FileDialog
-from calibre.utils.config import Config, ConfigProxy, dynamic, JSONConfig
+""" The GUI """
+
+import glob
+import os
+import signal
+import sys
+import threading
+from contextlib import contextmanager
+from threading import Lock, RLock
+
+from PyQt5.Qt import (
+    QT_VERSION, QApplication, QBuffer, QByteArray, QCoreApplication, QDateTime,
+    QDesktopServices, QDialog, QEvent, QFileDialog, QFileIconProvider, QFileInfo,
+    QFont, QFontDatabase, QFontInfo, QFontMetrics, QIcon, QLocale, QObject,
+    QSettings, QSocketNotifier, QStringListModel, Qt, QThread, QTimer, QTranslator,
+    QUrl, pyqtSignal
+)
+from PyQt5.QtWidgets import QStyle  # Gives a nicer error message than import from Qt
+
+from calibre import as_unicode, prints
+from calibre.constants import (
+    DEBUG, __appname__ as APP_UID, __version__, config_dir, filesystem_encoding,
+    is_running_from_develop, isbsd, isfrozen, islinux, isosx, iswindows, isxp,
+    plugins, plugins_loc
+)
 from calibre.ebooks.metadata import MetaInformation
+from calibre.gui2.linux_file_dialogs import (
+    check_for_linux_native_dialogs, linux_native_dialog
+)
+from calibre.gui2.qt_file_dialogs import FileDialog
+from calibre.ptempfile import base_dir
+from calibre.utils.config import Config, ConfigProxy, JSONConfig, dynamic
 from calibre.utils.date import UNDEFINED_DATE
-from calibre.utils.localization import get_lang
 from calibre.utils.file_type_icons import EXT_MAP
+from calibre.utils.localization import get_lang
+from polyglot.builtins import (iteritems, itervalues, unicode_type,
+        string_or_bytes, range, map)
+from polyglot import queue
 
 try:
     NO_URL_FORMATTING = QUrl.None_
 except AttributeError:
-    NO_URL_FORMATTING = QUrl.None
+    NO_URL_FORMATTING = getattr(QUrl, 'None')
+
 
 # Setup gprefs {{{
 gprefs = JSONConfig('gui')
@@ -168,6 +188,8 @@ def create_defs():
     defs['book_details_narrow_comments_layout'] = 'float'
     defs['book_list_split'] = False
     defs['wrap_toolbar_text'] = False
+    defs['dnd_merge'] = True
+    defs['booklist_grid'] = False
 
 
 create_defs()
@@ -303,7 +325,7 @@ def default_author_link():
 
 def available_heights():
     desktop  = QCoreApplication.instance().desktop()
-    return map(lambda x: x.height(), map(desktop.availableGeometry, range(desktop.screenCount())))
+    return list(map(lambda x: x.height(), map(desktop.availableGeometry, range(desktop.screenCount()))))
 
 
 def available_height():
@@ -336,7 +358,7 @@ def is_widescreen():
     global _is_widescreen
     if _is_widescreen is None:
         try:
-            _is_widescreen = float(available_width())/available_height() > 1.4
+            _is_widescreen = available_width()/available_height() > 1.4
         except:
             _is_widescreen = False
     return _is_widescreen
@@ -485,7 +507,7 @@ class FunctionDispatcher(QObject):
         if not queued:
             typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
         self.dispatch_signal.connect(self.dispatch, type=typ)
-        self.q = Queue.Queue()
+        self.q = queue.Queue()
         self.lock = threading.Lock()
 
     def __call__(self, *args, **kwargs):
@@ -552,7 +574,7 @@ class FileIconProvider(QFileIconProvider):
         upath, bpath = I('mimetypes'), I('mimetypes', allow_user_override=False)
         if upath != bpath:
             # User has chosen to override mimetype icons
-            path_map = {v:I('mimetypes/%s.png' % v) for v in set(self.ICONS.itervalues())}
+            path_map = {v:I('mimetypes/%s.png' % v) for v in set(itervalues(self.ICONS))}
             icons = self.ICONS.copy()
             for uicon in glob.glob(os.path.join(upath, '*.png')):
                 ukey = os.path.basename(uicon).rpartition('.')[0].lower()
@@ -560,18 +582,18 @@ class FileIconProvider(QFileIconProvider):
                     path_map[ukey] = uicon
                     icons[ukey] = ukey
         else:
-            path_map = {v:os.path.join(bpath, v + '.png') for v in set(self.ICONS.itervalues())}
+            path_map = {v:os.path.join(bpath, v + '.png') for v in set(itervalues(self.ICONS))}
             icons = self.ICONS
-        self.icons = {k:path_map[v] for k, v in icons.iteritems()}
+        self.icons = {k:path_map[v] for k, v in iteritems(icons)}
         self.icons['calibre'] = I('lt.png', allow_user_override=False)
         for i in ('dir', 'default', 'zero'):
             self.icons[i] = QIcon(self.icons[i])
 
     def key_from_ext(self, ext):
-        key = ext if ext in self.icons.keys() else 'default'
+        key = ext if ext in list(self.icons.keys()) else 'default'
         if key == 'default' and ext.count('.') > 0:
             ext = ext.rpartition('.')[2]
-            key = ext if ext in self.icons.keys() else 'default'
+            key = ext if ext in list(self.icons.keys()) else 'default'
         return key
 
     def cached_icon(self, key):
@@ -596,7 +618,7 @@ class FileIconProvider(QFileIconProvider):
         if fileinfo.isDir():
             key = 'dir'
         else:
-            ext = unicode(fileinfo.completeSuffix()).lower()
+            ext = unicode_type(fileinfo.completeSuffix()).lower()
             key = self.key_from_ext(ext)
         return self.cached_icon(key)
 
@@ -669,7 +691,7 @@ def pixmap_to_data(pixmap, format='JPEG', quality=None):
     buf = QBuffer(ba)
     buf.open(QBuffer.WriteOnly)
     pixmap.save(buf, format, quality=quality)
-    return bytes(ba.data())
+    return ba.data()
 
 
 def decouple(prefix):
@@ -716,9 +738,9 @@ class Translator(QTranslator):
 
     def translate(self, *args, **kwargs):
         try:
-            src = unicode(args[1])
+            src = unicode_type(args[1])
         except:
-            return u''
+            return ''
         t = _
         return t(src)
 
@@ -747,9 +769,9 @@ def load_builtin_fonts():
                 fid = QFontDatabase.addApplicationFontFromData(s.read())
                 if fid > -1:
                     fam = QFontDatabase.applicationFontFamilies(fid)
-                    fam = set(map(unicode, fam))
-                    if u'calibre Symbols' in fam:
-                        _rating_font = u'calibre Symbols'
+                    fam = set(map(unicode_type, fam))
+                    if 'calibre Symbols' in fam:
+                        _rating_font = 'calibre Symbols'
 
 
 def setup_gui_option_parser(parser):
@@ -794,7 +816,13 @@ class Application(QApplication):
 
     shutdown_signal_received = pyqtSignal()
 
-    def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs):
+    def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs, windows_app_uid=None):
+        if iswindows:
+            self.windows_app_uid = None
+            if windows_app_uid:
+                windows_app_uid = unicode_type(windows_app_uid)
+                if set_app_uid(windows_app_uid):
+                    self.windows_app_uid = windows_app_uid
         self.file_event_hook = None
         if isfrozen and QT_VERSION <= 0x050700 and 'wayland' in os.environ.get('QT_QPA_PLATFORM', ''):
             os.environ['QT_QPA_PLATFORM'] = 'xcb'
@@ -803,10 +831,12 @@ class Application(QApplication):
         if headless:
             if not args:
                 args = sys.argv[:1]
-            args.extend(['-platformpluginpath', sys.extensions_location, '-platform', 'headless'])
+            args.extend(['-platformpluginpath', plugins_loc, '-platform', 'headless'])
         self.headless = headless
-        qargs = [i.encode('utf-8') if isinstance(i, unicode) else i for i in args]
-        self.pi = plugins['progress_indicator'][0]
+        qargs = [i.encode('utf-8') if isinstance(i, unicode_type) else i for i in args]
+        self.pi, pi_err = plugins['progress_indicator']
+        if pi_err:
+            raise RuntimeError('Failed to load the progress_indicator C extension, with error: {}'.format(pi_err))
         if not isosx and not headless:
             # On OS X high dpi scaling is turned on automatically by the OS, so we dont need to set it explicitly
             setup_hidpi()
@@ -814,6 +844,8 @@ class Application(QApplication):
         QApplication.setOrganizationDomain(QApplication.organizationName())
         QApplication.setApplicationVersion(__version__)
         QApplication.setApplicationName(APP_UID)
+        if override_program_name and hasattr(QApplication, 'setDesktopFileName'):
+            QApplication.setDesktopFileName(override_program_name)
         QApplication.__init__(self, qargs)
         self.setAttribute(Qt.AA_UseHighDpiPixmaps)
         self.setAttribute(Qt.AA_SynthesizeTouchForUnhandledMouseEvents, False)
@@ -853,7 +885,7 @@ class Application(QApplication):
         self.line_height = max(12, QFontMetrics(self.font()).lineSpacing())
 
         dl = QLocale(get_lang())
-        if unicode(dl.bcp47Name()) != u'C':
+        if unicode_type(dl.bcp47Name()) != 'C':
             QLocale.setDefault(dl)
         global gui_thread, qt_app
         gui_thread = QThread.currentThread()
@@ -875,7 +907,7 @@ class Application(QApplication):
                 'calibre versions newer than 2.0 do not run on Windows XP. This is'
                 ' because the graphics toolkit calibre uses (Qt 5) crashes a lot'
                 ' on Windows XP. We suggest you stay with <a href="%s">calibre 1.48</a>'
-                ' which works well on Windows XP.') % 'http://download.calibre-ebook.com/1.48.0/', show=True)
+                ' which works well on Windows XP.') % 'https://download.calibre-ebook.com/1.48.0/', show=True)
             raise SystemExit(1)
 
         if iswindows:
@@ -891,6 +923,14 @@ class Application(QApplication):
             # Qt 5 bug: https://bugreports.qt-project.org/browse/QTBUG-41125
             self.aboutToQuit.connect(self.flush_clipboard)
 
+        if isosx:
+            cocoa, err = plugins['cocoa']
+            if err:
+                raise RuntimeError('Failed to load cocoa plugin with error: {}'.format(err))
+            cft = cocoa.cursor_blink_time()
+            if cft >= 0:
+                self.setCursorFlashTime(int(cft))
+
     def setup_ui_font(self):
         f = QFont(QApplication.font())
         q = (f.family(), f.pointSize())
@@ -941,24 +981,24 @@ class Application(QApplication):
     def load_calibre_style(self):
         icon_map = self.__icon_map_memory_ = {}
         pcache = {}
-        for k, v in {
-            'DialogYesButton': u'ok.png',
-            'DialogNoButton': u'window-close.png',
-            'DialogCloseButton': u'window-close.png',
-            'DialogOkButton': u'ok.png',
-            'DialogCancelButton': u'window-close.png',
-            'DialogHelpButton': u'help.png',
-            'DialogOpenButton': u'document_open.png',
-            'DialogSaveButton': u'save.png',
-            'DialogApplyButton': u'ok.png',
-            'DialogDiscardButton': u'trash.png',
-            'MessageBoxInformation': u'dialog_information.png',
-            'MessageBoxWarning': u'dialog_warning.png',
-            'MessageBoxCritical': u'dialog_error.png',
-            'MessageBoxQuestion': u'dialog_question.png',
-            'BrowserReload': u'view-refresh.png',
-            'LineEditClearButton': u'clear_left.png',
-        }.iteritems():
+        for k, v in iteritems({
+            'DialogYesButton': 'ok.png',
+            'DialogNoButton': 'window-close.png',
+            'DialogCloseButton': 'window-close.png',
+            'DialogOkButton': 'ok.png',
+            'DialogCancelButton': 'window-close.png',
+            'DialogHelpButton': 'help.png',
+            'DialogOpenButton': 'document_open.png',
+            'DialogSaveButton': 'save.png',
+            'DialogApplyButton': 'ok.png',
+            'DialogDiscardButton': 'trash.png',
+            'MessageBoxInformation': 'dialog_information.png',
+            'MessageBoxWarning': 'dialog_warning.png',
+            'MessageBoxCritical': 'dialog_error.png',
+            'MessageBoxQuestion': 'dialog_question.png',
+            'BrowserReload': 'view-refresh.png',
+            'LineEditClearButton': 'clear_left.png',
+        }):
             if v not in pcache:
                 p = I(v)
                 if isinstance(p, bytes):
@@ -983,7 +1023,7 @@ class Application(QApplication):
 
     def event(self, e):
         if callable(self.file_event_hook) and e.type() == QEvent.FileOpen:
-            path = unicode(e.file())
+            path = unicode_type(e.file())
             if os.access(path, os.R_OK):
                 with self._file_open_lock:
                     self._file_open_paths.append(path)
@@ -992,19 +1032,19 @@ class Application(QApplication):
         else:
             return QApplication.event(self, e)
 
-    @dynamic_property
+    @property
     def current_custom_colors(self):
+        from PyQt5.Qt import QColorDialog
+
+        return [col.getRgb() for col in
+                    (QColorDialog.customColor(i) for i in range(QColorDialog.customCount()))]
+
+    @current_custom_colors.setter
+    def current_custom_colors(self, colors):
         from PyQt5.Qt import QColorDialog, QColor
-
-        def fget(self):
-            return [col.getRgb() for col in
-                    (QColorDialog.customColor(i) for i in xrange(QColorDialog.customCount()))]
-
-        def fset(self, colors):
-            num = min(len(colors), QColorDialog.customCount())
-            for i in xrange(num):
-                QColorDialog.setCustomColor(i, QColor(*colors[i]))
-        return property(fget=fget, fset=fset)
+        num = min(len(colors), QColorDialog.customCount())
+        for i in range(num):
+            QColorDialog.setCustomColor(i, QColor(*colors[i]))
 
     def read_custom_colors(self):
         colors = self.color_prefs.get('custom_colors_for_color_dialog', None)
@@ -1025,11 +1065,15 @@ class Application(QApplication):
 
     def setup_unix_signals(self):
         import fcntl
-        read_fd, write_fd = os.pipe()
-        cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
-        for fd in (read_fd, write_fd):
-            flags = fcntl.fcntl(fd, fcntl.F_GETFD)
-            fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag | os.O_NONBLOCK)
+        if hasattr(os, 'pipe2'):
+            read_fd, write_fd = os.pipe2(os.O_CLOEXEC | os.O_NONBLOCK)
+        else:
+            read_fd, write_fd = os.pipe()
+            cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
+            for fd in (read_fd, write_fd):
+                flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+                fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag | os.O_NONBLOCK)
+
         for sig in (signal.SIGINT, signal.SIGTERM):
             signal.signal(sig, lambda x, y: None)
             signal.siginterrupt(sig, False)
@@ -1066,7 +1110,7 @@ def sanitize_env_vars():
 
     originals = {x:os.environ.get(x, '') for x in env_vars}
     changed = {x:False for x in env_vars}
-    for var, suffix in env_vars.iteritems():
+    for var, suffix in iteritems(env_vars):
         paths = [x for x in originals[var].split(os.pathsep) if x]
         npaths = [] if suffix is None else [x for x in paths if x != (sys.frozen_path + suffix)]
         if len(npaths) < len(paths):
@@ -1079,7 +1123,7 @@ def sanitize_env_vars():
     try:
         yield
     finally:
-        for var, orig in originals.iteritems():
+        for var, orig in iteritems(originals):
             if changed[var]:
                 if orig:
                     os.environ[var] = orig
@@ -1094,12 +1138,24 @@ def open_url(qurl):
     # Qt 5 requires QApplication to be constructed before trying to use
     # QDesktopServices::openUrl()
     ensure_app()
-    if isinstance(qurl, basestring):
+    if isinstance(qurl, string_or_bytes):
         qurl = QUrl(qurl)
     with sanitize_env_vars():
         QDesktopServices.openUrl(qurl)
 
 
+def safe_open_url(qurl):
+    if isinstance(qurl, string_or_bytes):
+        qurl = QUrl(qurl)
+    if qurl.scheme() in ('', 'file'):
+        path = qurl.toLocalFile()
+        ext = os.path.splitext(path)[-1].lower()[1:]
+        if ext in ('exe', 'com', 'cmd', 'bat', 'sh', 'psh', 'ps1', 'vbs', 'js', 'wsf', 'vba', 'py', 'rb', 'pl'):
+            prints('Refusing to open file:', path)
+            return
+    open_url(qurl)
+
+
 def get_current_db():
     '''
     This method will try to return the current database in use by the user as
@@ -1133,7 +1189,7 @@ def ensure_app(headless=True):
             args = sys.argv[:1]
             has_headless = isosx or islinux or isbsd
             if headless and has_headless:
-                args += ['-platformpluginpath', sys.extensions_location, '-platform', 'headless']
+                args += ['-platformpluginpath', plugins_loc, '-platform', 'headless']
             _store_app = QApplication(args)
             if headless and has_headless:
                 _store_app.headless = True
@@ -1202,7 +1258,7 @@ def elided_text(text, font=None, width=300, pos='middle'):
     from PyQt5.Qt import QFontMetrics, QApplication
     fm = QApplication.fontMetrics() if font is None else (font if isinstance(font, QFontMetrics) else QFontMetrics(font))
     delta = 4
-    ellipsis = u'\u2026'
+    ellipsis = '\u2026'
 
     def remove_middle(x):
         mid = len(x) // 2
@@ -1211,7 +1267,7 @@ def elided_text(text, font=None, width=300, pos='middle'):
     chomp = {'middle':remove_middle, 'left':lambda x:(ellipsis + x[delta:]), 'right':lambda x:(x[:-delta] + ellipsis)}[pos]
     while len(text) > delta and fm.width(text) > width:
         text = chomp(text)
-    return unicode(text)
+    return unicode_type(text)
 
 
 def find_forms(srcdir):
@@ -1230,8 +1286,9 @@ def form_to_compiled_form(form):
 
 
 def build_forms(srcdir, info=None, summary=False, check_for_migration=False):
-    import re, cStringIO
+    import re
     from PyQt5.uic import compileUi
+    from polyglot.io import PolyglotStringIO
     forms = find_forms(srcdir)
     if info is None:
         from calibre import prints
@@ -1255,7 +1312,7 @@ def build_forms(srcdir, info=None, summary=False, check_for_migration=False):
         if force_compile or not os.path.exists(compiled_form) or os.stat(form).st_mtime > os.stat(compiled_form).st_mtime:
             if not summary:
                 info('\tCompiling form', form)
-            buf = cStringIO.StringIO()
+            buf = PolyglotStringIO()
             compileUi(form, buf)
             dat = buf.getvalue()
             dat = dat.replace('import images_rc', '')
@@ -1264,7 +1321,8 @@ def build_forms(srcdir, info=None, summary=False, check_for_migration=False):
             dat = dat.replace('_("MMM yyyy")', '"MMM yyyy"')
             dat = dat.replace('_("d MMM yyyy")', '"d MMM yyyy"')
             dat = pat.sub(sub, dat)
-
+            if not isinstance(dat, bytes):
+                dat = dat.encode('utf-8')
             open(compiled_form, 'wb').write(dat)
             num += 1
     if num:
@@ -1280,7 +1338,7 @@ if is_running_from_develop:
 def event_type_name(ev_or_etype):
     from PyQt5.QtCore import QEvent
     etype = ev_or_etype.type() if isinstance(ev_or_etype, QEvent) else ev_or_etype
-    for name, num in vars(QEvent).iteritems():
+    for name, num in iteritems(vars(QEvent)):
         if num == etype:
             return name
     return 'UnknownEventType'
@@ -1307,25 +1365,6 @@ empty_model = QStringListModel([''])
 empty_index = empty_model.index(0)
 
 
-def get_app_uid():
-    import ctypes
-    from ctypes import wintypes
-    lpBuffer = wintypes.LPWSTR()
-    try:
-        AppUserModelID = ctypes.windll.shell32.GetCurrentProcessExplicitAppUserModelID
-    except Exception:  # Vista has no app uids
-        return
-    AppUserModelID.argtypes = [wintypes.LPWSTR]
-    AppUserModelID.restype = wintypes.HRESULT
-    try:
-        AppUserModelID(ctypes.cast(ctypes.byref(lpBuffer), wintypes.LPWSTR))
-    except Exception:
-        return
-    appid = lpBuffer.value
-    ctypes.windll.ole32.CoTaskMemFree(lpBuffer)
-    return appid
-
-
 def set_app_uid(val):
     import ctypes
     from ctypes import wintypes
@@ -1336,19 +1375,13 @@ def set_app_uid(val):
     AppUserModelID.argtypes = [wintypes.LPCWSTR]
     AppUserModelID.restype = wintypes.HRESULT
     try:
-        AppUserModelID(unicode(val))
+        AppUserModelID(unicode_type(val))
     except Exception as err:
-        prints(u'Failed to set app uid with error:', as_unicode(err))
+        prints('Failed to set app uid with error:', as_unicode(err))
         return False
     return True
 
 
 def add_to_recent_docs(path):
-    from win32com.shell import shell, shellcon
-    path = unicode(path)
-    app_id = get_app_uid()
-    if app_id is None:
-        shell.SHAddToRecentDocs(shellcon.SHARD_PATHW, path)
-    else:
-        item = shell.SHCreateItemFromParsingName(path, None, shell.IID_IShellItem)
-        shell.SHAddToRecentDocs(shellcon.SHARD_APPIDINFO, (item, app_id))
+    app = QApplication.instance()
+    plugins['winutil'][0].add_to_recent_docs(unicode_type(path), app.windows_app_uid)
