@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -14,7 +15,7 @@ from calibre.gui2.tools import generate_catalog
 from calibre.utils.config import dynamic
 from calibre.gui2.actions import InterfaceAction
 from calibre import sanitize_file_name
-from polyglot.builtins import range
+from polyglot.builtins import range, map
 
 
 class GenerateCatalogAction(InterfaceAction):
@@ -22,7 +23,7 @@ class GenerateCatalogAction(InterfaceAction):
     name = 'Generate Catalog'
     action_spec = (_('Create catalog'), 'catalog.png',
                    _('Create a catalog of the books in your calibre library in different formats'), ())
-    dont_add_to = frozenset(['context-menu-device'])
+    dont_add_to = frozenset(('context-menu-device',))
 
     def genesis(self):
         self.qaction.triggered.connect(self.generate_catalog)
@@ -36,7 +37,7 @@ class GenerateCatalogAction(InterfaceAction):
         rows = self.gui.library_view.selectionModel().selectedRows()
         if not rows or len(rows) < 2:
             rows = range(self.gui.library_view.model().rowCount(QModelIndex()))
-        ids = map(self.gui.library_view.model().id, rows)
+        ids = list(map(self.gui.library_view.model().id, rows))
 
         if not ids:
             return error_dialog(self.gui, _('No books selected'),
@@ -85,7 +86,7 @@ class GenerateCatalogAction(InterfaceAction):
             id = self.gui.library_view.model().add_catalog(job.catalog_file_path, job.catalog_title)
             self.gui.library_view.model().beginResetModel(), self.gui.library_view.model().endResetModel()
             if job.catalog_sync:
-                sync = dynamic.get('catalogs_to_be_synced', set([]))
+                sync = dynamic.get('catalogs_to_be_synced', set())
                 sync.add(id)
                 dynamic.set('catalogs_to_be_synced', sync)
         self.gui.status_bar.show_message(_('Catalog generated.'), 3000)
