@@ -1,9 +1,10 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 # License: GPLv3 Copyright: 2010, Kovid Goyal <kovid at kovidgoyal.net>
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-import cPickle, re
-from binascii import unhexlify
+import os
+import re
 from collections import namedtuple
 from functools import partial
 
@@ -23,8 +24,8 @@ from calibre.ebooks.metadata.search_internet import (
     url_for_book_search
 )
 from calibre.gui2 import (
-    NO_URL_FORMATTING, config, default_author_link, gprefs, open_url, pixmap_to_data,
-    rating_font
+    NO_URL_FORMATTING, choose_save_file, config, default_author_link, gprefs,
+    safe_open_url, pixmap_to_data, rating_font
 )
 from calibre.gui2.dnd import (
     dnd_get_files, dnd_get_image, dnd_has_extension, dnd_has_image, image_extensions
@@ -32,18 +33,34 @@ from calibre.gui2.dnd import (
 from calibre.utils.config import tweaks
 from calibre.utils.img import blend_image, image_from_x
 from calibre.utils.localization import is_rtl
+from calibre.utils.serialize import json_loads
+from polyglot.builtins import unicode_type
+from polyglot.binary import from_hex_bytes, from_hex_unicode
+
 
 _css = None
 InternetSearch = namedtuple('InternetSearch', 'author where')
 
 
+def set_html(mi, html, web_view):
+    from calibre.gui2.ui import get_gui
+    gui = get_gui()
+    book_id = getattr(mi, 'id', None)
+    if gui and book_id is not None:
+        path = gui.current_db.abspath(book_id, index_is_id=True)
+        if path:
+            web_view.setHtml(html, QUrl.fromLocalFile(os.path.join(path, 'metadata.html')))
+            return
+    web_view.setHtml(html)
+
+
 def css():
     global _css
     if _css is None:
         val = P('templates/book_details.css', data=True).decode('utf-8')
         col = QApplication.instance().palette().color(QPalette.Link).name()
         val = val.replace('LINK_COLOR', col)
-        _css = re.sub(ur'/\*.*?\*/', '', val, flags=re.DOTALL)
+        _css = re.sub(unicode_type(r'/\*.*?\*/'), '', val, flags=re.DOTALL)
     return _css
 
 
@@ -86,27 +103,32 @@ def init_manage_action(ac, field, value):
     return ac
 
 
-def render_html(mi, css, vertical, widget, all_fields=False, render_data_func=None):  # {{{
-    table, comment_fields = (render_data_func or render_data)(mi, all_fields=all_fields,
-            use_roman_numbers=config['use_roman_numerals_for_series_number'])
+def render_html(mi, css, vertical, widget, all_fields=False, render_data_func=None, pref_name='book_display_fields'):  # {{{
+    func = render_data_func or render_data
+    try:
+        table, comment_fields = func(mi, all_fields=all_fields,
+                use_roman_numbers=config['use_roman_numerals_for_series_number'], pref_name=pref_name)
+    except TypeError:
+        table, comment_fields = func(mi, all_fields=all_fields,
+                use_roman_numbers=config['use_roman_numerals_for_series_number'])
 
     def color_to_string(col):
         ans = '#000000'
         if col.isValid():
             col = col.toRgb()
             if col.isValid():
-                ans = unicode(col.name())
+                ans = unicode_type(col.name())
         return ans
 
     fi = QFontInfo(QApplication.font(widget))
     f = fi.pixelSize() + 1 + int(tweaks['change_book_details_font_size_by'])
-    fam = unicode(fi.family()).strip().replace('"', '')
+    fam = unicode_type(fi.family()).strip().replace('"', '')
     if not fam:
         fam = 'sans-serif'
 
     c = color_to_string(QApplication.palette().color(QPalette.Normal,
                     QPalette.WindowText))
-    templ = u'''\
+    templ = '''\
     <html>
         <head>
         <style type="text/css">
@@ -126,47 +148,45 @@ def render_html(mi, css, vertical, widget, all_fields=False, render_data_func=No
         </body>
     <html>
     '''%(f, fam, c, css)
-    comments = u''
+    comments = ''
     if comment_fields:
         comments = '\n'.join(u'<div>%s</div>' % x for x in comment_fields)
-    right_pane = u'<div id="comments" class="comments">%s</div>'%comments
+    right_pane = '<div id="comments" class="comments">%s</div>'%comments
 
     if vertical:
         ans = templ%(table+right_pane)
     else:
         if gprefs['book_details_narrow_comments_layout'] == 'columns':
-            ans = templ%(u'<table><tr><td valign="top" '
+            ans = templ%('<table><tr><td valign="top" '
                 'style="padding-right:2em; width:40%%">%s</td><td valign="top">%s</td></tr></table>'
                     % (table, right_pane))
         else:
-            ans = templ%(u'<div style="float: left; margin-right: 1em; margin-bottom: 1em; max-width: 40%">{}</div><div>{}</div>'.format(
+            ans = templ%('<div style="float: left; margin-right: 1em; margin-bottom: 1em; max-width: 40%">{}</div><div>{}</div>'.format(
                     table, right_pane))
     return ans
 
 
-def get_field_list(fm, use_defaults=False):
+def get_field_list(fm, use_defaults=False, pref_name='book_display_fields'):
     from calibre.gui2.ui import get_gui
     db = get_gui().current_db
     if use_defaults:
         src = db.prefs.defaults
     else:
-        old_val = gprefs.get('book_display_fields', None)
-        if old_val is not None and not db.prefs.has_setting(
-                'book_display_fields'):
+        old_val = gprefs.get(pref_name, None)
+        if old_val is not None and not db.prefs.has_setting(pref_name):
             src = gprefs
         else:
             src = db.prefs
-    fieldlist = list(src['book_display_fields'])
-    names = frozenset([x[0] for x in fieldlist])
-    for field in fm.displayable_field_keys():
-        if field not in names:
-            fieldlist.append((field, True))
+    fieldlist = list(src[pref_name])
+    names = frozenset(x[0] for x in fieldlist)
     available = frozenset(fm.displayable_field_keys())
+    for field in available - names:
+        fieldlist.append((field, True))
     return [(f, d) for f, d in fieldlist if f in available]
 
 
-def render_data(mi, use_roman_numbers=True, all_fields=False):
-    field_list = get_field_list(getattr(mi, 'field_metadata', field_metadata))
+def render_data(mi, use_roman_numbers=True, all_fields=False, pref_name='book_display_fields'):
+    field_list = get_field_list(getattr(mi, 'field_metadata', field_metadata), pref_name=pref_name)
     field_list = [(x, all_fields or display) for x, display in field_list]
     return mi_to_html(mi, field_list=field_list, use_roman_numbers=use_roman_numbers, rtl=is_rtl(),
                       rating_font=rating_font(), default_author_link=default_author_link())
@@ -178,7 +198,7 @@ def details_context_menu_event(view, ev, book_info):  # {{{
     p = view.page()
     mf = p.mainFrame()
     r = mf.hitTestContent(ev.pos())
-    url = unicode(r.linkUrl().toString(NO_URL_FORMATTING)).strip()
+    url = unicode_type(r.linkUrl().toString(NO_URL_FORMATTING)).strip()
     menu = p.createStandardContextMenu()
     ca = view.pageAction(p.Copy)
     for action in list(menu.actions()):
@@ -187,6 +207,7 @@ def details_context_menu_event(view, ev, book_info):  # {{{
     menu.addAction(QIcon(I('edit-copy.png')), _('Copy &all'), partial(copy_all, book_info))
     search_internet_added = False
     if not r.isNull():
+        from calibre.ebooks.oeb.polish.main import SUPPORTED
         if url.startswith('format:'):
             parts = url.split(':')
             try:
@@ -196,7 +217,6 @@ def details_context_menu_event(view, ev, book_info):  # {{{
                 traceback.print_exc()
             else:
                 from calibre.gui2.ui import get_gui
-                from calibre.ebooks.oeb.polish.main import SUPPORTED
                 db = get_gui().current_db.new_api
                 ofmt = fmt.upper() if fmt.startswith('ORIGINAL_') else 'ORIGINAL_' + fmt
                 nfmt = ofmt[len('ORIGINAL_'):]
@@ -223,7 +243,11 @@ def details_context_menu_event(view, ev, book_info):  # {{{
                 if not fmt.upper().startswith('ORIGINAL_'):
                     from calibre.gui2.open_with import populate_menu, edit_programs
                     m = QMenu(_('Open %s with...') % fmt.upper())
-                    populate_menu(m, partial(book_info.open_with, book_id, fmt), fmt)
+
+                    def connect_action(ac, entry):
+                        connect_lambda(ac.triggered, book_info, lambda book_info: book_info.open_with(book_id, fmt, entry))
+
+                    populate_menu(m, connect_action, fmt)
                     if len(m.actions()) == 0:
                         menu.addAction(_('Open %s with...') % fmt.upper(), partial(book_info.choose_open_with, book_id, fmt))
                     else:
@@ -232,6 +256,9 @@ def details_context_menu_event(view, ev, book_info):  # {{{
                         m.addAction(_('Edit Open With applications...'), partial(edit_programs, fmt, book_info))
                         menu.addMenu(m)
                         menu.ow = m
+                    if fmt.upper() in SUPPORTED:
+                        menu.addSeparator()
+                        menu.addAction(_('Edit %s...') % fmt.upper(), partial(book_info.edit_fmt, book_id, fmt))
                 ac = book_info.copy_link_action
                 ac.current_url = r.linkElement().attribute('data-full-path')
                 if ac.current_url:
@@ -240,7 +267,7 @@ def details_context_menu_event(view, ev, book_info):  # {{{
         else:
             el = r.linkElement()
             data = el.attribute('data-item')
-            author = el.toPlainText() if unicode(el.attribute('calibre-data')) == u'authors' else None
+            author = el.toPlainText() if unicode_type(el.attribute('calibre-data')) == 'authors' else None
             if url and not url.startswith('search:'):
                 for a, t in [('copy', _('&Copy link')),
                 ]:
@@ -261,13 +288,15 @@ def details_context_menu_event(view, ev, book_info):  # {{{
                                    lambda : book_info.search_requested('authors:"={}"'.format(author.replace('"', r'\"'))))
             if data:
                 try:
-                    field, value, book_id = cPickle.loads(unhexlify(data))
+                    field, value, book_id = json_loads(from_hex_bytes(data))
                 except Exception:
                     field = value = book_id = None
                 if field:
-                    if author is None and (
-                            field in ('tags', 'series', 'publisher') or is_category(field)):
-                        menu.addAction(init_manage_action(book_info.manage_action, field, value))
+                    if author is None:
+                        if field in ('tags', 'series', 'publisher') or is_category(field):
+                            menu.addAction(init_manage_action(book_info.manage_action, field, value))
+                        elif field == 'identifiers':
+                            menu.addAction(book_info.edit_identifiers_action)
                     ac = book_info.remove_item_action
                     ac.data = (field, value, book_id)
                     ac.setText(_('Remove %s from this book') % value)
@@ -354,12 +383,12 @@ class CoverView(QWidget):  # {{{
         extrax = canvas_size.width() - width
         if extrax < 0:
             extrax = 0
-        x = int(extrax/2.)
+        x = int(extrax//2)
         height = self.current_pixmap_size.height()
         extray = canvas_size.height() - height
         if extray < 0:
             extray = 0
-        y = int(extray/2.)
+        y = int(extray//2)
         target = QRect(x, y, width, height)
         p = QPainter(self)
         p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
@@ -375,7 +404,7 @@ class CoverView(QWidget):  # {{{
             f = p.font()
             f.setBold(True)
             p.setFont(f)
-            sz = u'\u00a0%d x %d\u00a0'%(self.pixmap.width(), self.pixmap.height())
+            sz = '\u00a0%d x %d\u00a0'%(self.pixmap.width(), self.pixmap.height())
             flags = Qt.AlignBottom|Qt.AlignRight|Qt.TextSingleLine
             szrect = p.boundingRect(sztgt, flags, sz)
             p.fillRect(szrect.adjusted(0, 0, 0, 4), QColor(0, 0, 0, 200))
@@ -393,6 +422,7 @@ class CoverView(QWidget):  # {{{
         cm = QMenu(self)
         paste = cm.addAction(_('Paste cover'))
         copy = cm.addAction(_('Copy cover'))
+        save = cm.addAction(_('Save cover to disk'))
         remove = cm.addAction(_('Remove cover'))
         gc = cm.addAction(_('Generate cover from metadata'))
         cm.addSeparator()
@@ -402,9 +432,14 @@ class CoverView(QWidget):  # {{{
         paste.triggered.connect(self.paste_from_clipboard)
         remove.triggered.connect(self.remove_cover)
         gc.triggered.connect(self.generate_cover)
+        save.triggered.connect(self.save_cover)
 
         m = QMenu(_('Open cover with...'))
-        populate_menu(m, self.open_with, 'cover_image')
+
+        def connect_action(ac, entry):
+            connect_lambda(ac.triggered, self, lambda self: self.open_with(entry))
+
+        populate_menu(m, connect_action, 'cover_image')
         if len(m.actions()) == 0:
             cm.addAction(_('Open cover with...'), self.choose_open_with)
         else:
@@ -440,6 +475,18 @@ class CoverView(QWidget):  # {{{
         if not pmap.isNull():
             self.update_cover(pmap)
 
+    def save_cover(self):
+        from calibre.gui2.ui import get_gui
+        book_id = self.data.get('id')
+        db = get_gui().current_db.new_api
+        path = choose_save_file(
+            self, 'save-cover-from-book-details', _('Choose cover save location'),
+            filters=[(_('JPEG images'), ['jpg', 'jpeg'])], all_files=False,
+            initial_filename='{}.jpeg'.format(db.field_for('title', book_id, default_value='cover'))
+        )
+        if path:
+            db.copy_cover_to(book_id, path)
+
     def update_cover(self, pmap=None, cdata=None):
         if pmap is None:
             pmap = QPixmap()
@@ -504,6 +551,8 @@ class BookInfo(QWebView):
     copy_link = pyqtSignal(object)
     manage_category = pyqtSignal(object, object)
     open_fmt_with = pyqtSignal(int, object, object)
+    edit_book = pyqtSignal(int, object)
+    edit_identifiers = pyqtSignal()
 
     def __init__(self, vertical, parent=None):
         QWebView.__init__(self, parent)
@@ -532,6 +581,8 @@ class BookInfo(QWebView):
         self.manage_action = QAction(self)
         self.manage_action.current_fmt = self.manage_action.current_url = None
         self.manage_action.triggered.connect(self.manage_action_triggered)
+        self.edit_identifiers_action = QAction(QIcon(I('identifiers.png')), _('Edit identifiers for this book'), self)
+        self.edit_identifiers_action.triggered.connect(self.edit_identifiers)
         self.remove_item_action = ac = QAction(QIcon(I('minus.png')), '...', self)
         ac.data = (None, None, None)
         ac.triggered.connect(self.remove_item_triggered)
@@ -575,9 +626,9 @@ class BookInfo(QWebView):
 
     def link_activated(self, link):
         self._link_clicked = True
-        if unicode(link.scheme()) in ('http', 'https'):
-            return open_url(link)
-        link = unicode(link.toString(NO_URL_FORMATTING))
+        if unicode_type(link.scheme()) in ('http', 'https'):
+            return safe_open_url(link)
+        link = unicode_type(link.toString(NO_URL_FORMATTING))
         self.link_clicked.emit(link)
 
     def turnoff_scrollbar(self, *args):
@@ -585,7 +636,7 @@ class BookInfo(QWebView):
 
     def show_data(self, mi):
         html = render_html(mi, css(), self.vertical, self.parent())
-        self.setHtml(html)
+        set_html(mi, html, self)
 
     def mouseDoubleClickEvent(self, ev):
         swidth = self.page().mainFrame().scrollBarGeometry(Qt.Vertical).width()
@@ -609,6 +660,9 @@ class BookInfo(QWebView):
         if entry is not None:
             self.open_with(book_id, fmt, entry)
 
+    def edit_fmt(self, book_id, fmt):
+        self.edit_book.emit(book_id, fmt)
+
 
 # }}}
 
@@ -657,7 +711,7 @@ class DetailsLayout(QLayout):  # {{{
     def cover_height(self, r):
         if not self._children[0].widget().isVisible():
             return 0
-        mh = min(int(r.height()/2.), int(4/3. * r.width())+1)
+        mh = min(int(r.height()//2), int(4/3 * r.width())+1)
         try:
             ph = self._children[0].widget().pixmap.height()
         except:
@@ -669,7 +723,7 @@ class DetailsLayout(QLayout):  # {{{
     def cover_width(self, r):
         if not self._children[0].widget().isVisible():
             return 0
-        mw = 1 + int(3/4. * r.height())
+        mw = 1 + int(3/4 * r.height())
         try:
             pw = self._children[0].widget().pixmap.width()
         except:
@@ -722,7 +776,9 @@ class BookDetails(QWidget):  # {{{
     cover_removed = pyqtSignal(object)
     view_device_book = pyqtSignal(object)
     manage_category = pyqtSignal(object, object)
+    edit_identifiers = pyqtSignal()
     open_fmt_with = pyqtSignal(int, object, object)
+    edit_book = pyqtSignal(int, object)
 
     # Drag 'n drop {{{
 
@@ -792,12 +848,14 @@ class BookDetails(QWidget):  # {{{
         self.book_info.remove_format.connect(self.remove_specific_format)
         self.book_info.remove_item.connect(self.remove_metadata_item)
         self.book_info.open_fmt_with.connect(self.open_fmt_with)
+        self.book_info.edit_book.connect(self.edit_book)
         self.book_info.save_format.connect(self.save_specific_format)
         self.book_info.restore_format.connect(self.restore_specific_format)
         self.book_info.set_cover_format.connect(self.set_cover_from_format)
         self.book_info.compare_format.connect(self.compare_specific_format)
         self.book_info.copy_link.connect(self.copy_link)
         self.book_info.manage_category.connect(self.manage_category)
+        self.book_info.edit_identifiers.connect(self.edit_identifiers)
         self.setCursor(Qt.PointingHandCursor)
 
     def search_internet(self, data):
@@ -806,7 +864,7 @@ class BookDetails(QWidget):  # {{{
                 url = url_for_book_search(data.where, title=self.last_data['title'], author=self.last_data['authors'][0])
             else:
                 url = url_for_author_search(data.where, author=data.author)
-            open_url(url)
+            safe_open_url(url)
 
     def handle_click(self, link):
         typ, val = link.partition(':')[0::2]
@@ -818,10 +876,10 @@ class BookDetails(QWidget):  # {{{
         elif typ == 'devpath':
             self.view_device_book.emit(val)
         elif typ == 'search':
-            self.search_requested.emit(unhexlify(val).decode('utf-8'))
+            self.search_requested.emit(from_hex_unicode(val))
         else:
             try:
-                open_url(QUrl(link, QUrl.TolerantMode))
+                safe_open_url(QUrl(link, QUrl.TolerantMode))
             except:
                 import traceback
                 traceback.print_exc()
@@ -837,7 +895,7 @@ class BookDetails(QWidget):  # {{{
             self.last_data = {}
         self.book_info.show_data(data)
         self.cover_view.show_data(data)
-        self.current_path = getattr(data, u'path', u'')
+        self.current_path = getattr(data, 'path', '')
         self.update_layout()
 
     def update_layout(self):
