@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -24,9 +25,10 @@ from calibre.ebooks.chardet import xml_to_unicode
 from calibre import xml_replace_entities, prepare_string_for_xml
 from calibre.gui2 import open_url, error_dialog, choose_files, gprefs, NO_URL_FORMATTING, secure_web_page
 from calibre.gui2.widgets import LineEditECM
-from calibre.utils.soupparser import fromstring
+from html5_parser import parse
 from calibre.utils.config import tweaks
 from calibre.utils.imghdr import what
+from polyglot.builtins import unicode_type
 
 
 class PageAction(QAction):  # {{{
@@ -214,13 +216,13 @@ class EditorWidget(QWebView, LineEditECM):  # {{{
         col = QColorDialog.getColor(Qt.black, self,
                 _('Choose foreground color'), QColorDialog.ShowAlphaChannel)
         if col.isValid():
-            self.exec_command('foreColor', unicode(col.name()))
+            self.exec_command('foreColor', unicode_type(col.name()))
 
     def background_color(self):
         col = QColorDialog.getColor(Qt.white, self,
                 _('Choose background color'), QColorDialog.ShowAlphaChannel)
         if col.isValid():
-            self.exec_command('hiliteColor', unicode(col.name()))
+            self.exec_command('hiliteColor', unicode_type(col.name()))
 
     def insert_hr(self, *args):
         self.exec_command('insertHTML', '<hr>')
@@ -231,7 +233,7 @@ class EditorWidget(QWebView, LineEditECM):  # {{{
             return
         url = self.parse_link(link)
         if url.isValid():
-            url = unicode(url.toString(NO_URL_FORMATTING))
+            url = unicode_type(url.toString(NO_URL_FORMATTING))
             self.setFocus(Qt.OtherFocusReason)
             if is_image:
                 self.exec_command('insertHTML',
@@ -293,7 +295,7 @@ class EditorWidget(QWebView, LineEditECM):  # {{{
         d.resize(d.sizeHint())
         link, name, is_image = None, None, False
         if d.exec_() == d.Accepted:
-            link, name = unicode(d.url.text()).strip(), unicode(d.name.text()).strip()
+            link, name = unicode_type(d.url.text()).strip(), unicode_type(d.name.text()).strip()
             is_image = d.treat_as_image.isChecked()
         return link, name, is_image
 
@@ -327,7 +329,7 @@ class EditorWidget(QWebView, LineEditECM):  # {{{
         frame = self.page().mainFrame()
         if arg is not None:
             js = 'document.execCommand("%s", false, %s);' % (cmd,
-                    json.dumps(unicode(arg)))
+                    json.dumps(unicode_type(arg)))
         else:
             js = 'document.execCommand("%s", false, null);' % cmd
         frame.evaluateJavaScript(js)
@@ -335,55 +337,53 @@ class EditorWidget(QWebView, LineEditECM):  # {{{
     def remove_format_cleanup(self):
         self.html = self.html
 
-    @dynamic_property
+    @property
     def html(self):
+        ans = ''
+        try:
+            if not self.page().mainFrame().documentElement().findFirst('meta[name="calibre-dont-sanitize"]').isNull():
+                # Bypass cleanup if special meta tag exists
+                return unicode_type(self.page().mainFrame().toHtml())
+            check = unicode_type(self.page().mainFrame().toPlainText()).strip()
+            raw = unicode_type(self.page().mainFrame().toHtml())
+            raw = xml_to_unicode(raw, strip_encoding_pats=True,
+                                resolve_entities=True)[0]
+            raw = self.comments_pat.sub('', raw)
+            if not check and '<img' not in raw.lower():
+                return ans
 
-        def fget(self):
-            ans = u''
             try:
-                if not self.page().mainFrame().documentElement().findFirst('meta[name="calibre-dont-sanitize"]').isNull():
-                    # Bypass cleanup if special meta tag exists
-                    return unicode(self.page().mainFrame().toHtml())
-                check = unicode(self.page().mainFrame().toPlainText()).strip()
-                raw = unicode(self.page().mainFrame().toHtml())
-                raw = xml_to_unicode(raw, strip_encoding_pats=True,
-                                    resolve_entities=True)[0]
-                raw = self.comments_pat.sub('', raw)
-                if not check and '<img' not in raw.lower():
-                    return ans
+                root = html.fromstring(raw)
+            except Exception:
+                root = parse(raw, maybe_xhtml=False, sanitize_names=True)
 
-                try:
-                    root = html.fromstring(raw)
-                except:
-                    root = fromstring(raw)
+            elems = []
+            for body in root.xpath('//body'):
+                if body.text:
+                    elems.append(body.text)
+                elems += [html.tostring(x, encoding='unicode') for x in body if
+                    x.tag not in ('script', 'style')]
 
-                elems = []
-                for body in root.xpath('//body'):
-                    if body.text:
-                        elems.append(body.text)
-                    elems += [html.tostring(x, encoding=unicode) for x in body if
-                        x.tag not in ('script', 'style')]
-
-                if len(elems) > 1:
-                    ans = u'<div>%s</div>'%(u''.join(elems))
-                else:
-                    ans = u''.join(elems)
-                    if not ans.startswith('<'):
-                        ans = '<p>%s</p>'%ans
-                ans = xml_replace_entities(ans)
-            except:
-                import traceback
-                traceback.print_exc()
-
-            return ans
-
-        def fset(self, val):
-            if self.base_url is None:
-                self.setHtml(val)
+            if len(elems) > 1:
+                ans = '<div>%s</div>'%(''.join(elems))
             else:
-                self.setHtml(val, self.base_url)
-            self.set_font_style()
-        return property(fget=fget, fset=fset)
+                ans = ''.join(elems)
+                if not ans.startswith('<'):
+                    ans = '<p>%s</p>'%ans
+            ans = xml_replace_entities(ans)
+        except:
+            import traceback
+            traceback.print_exc()
+
+        return ans
+
+    @html.setter
+    def html(self, val):
+        if self.base_url is None:
+            self.setHtml(val)
+        else:
+            self.setHtml(val, self.base_url)
+        self.set_font_style()
 
     def set_base_url(self, qurl):
         self.base_url = qurl
@@ -395,13 +395,13 @@ class EditorWidget(QWebView, LineEditECM):  # {{{
             return
         mf = self.page().mainFrame()
         mf.evaluateJavaScript('document.execCommand("selectAll", false, null)')
-        mf.evaluateJavaScript('document.execCommand("insertHTML", false, %s)' % json.dumps(unicode(val)))
+        mf.evaluateJavaScript('document.execCommand("insertHTML", false, %s)' % json.dumps(unicode_type(val)))
         self.set_font_style()
 
     def set_font_style(self):
         fi = QFontInfo(QApplication.font(self))
         f  = fi.pixelSize() + 1 + int(tweaks['change_book_details_font_size_by'])
-        fam = unicode(fi.family()).strip().replace('"', '')
+        fam = unicode_type(fi.family()).strip().replace('"', '')
         if not fam:
             fam = 'sans-serif'
         style = 'font-size: %fpx; font-family:"%s",sans-serif;' % (f, fam)
@@ -483,7 +483,7 @@ class Highlighter(QSyntaxHighlighter):
             if state == State_Comment:
                 start = pos
                 while pos < len_:
-                    if text[pos:pos+3] == u"-->":
+                    if text[pos:pos+3] == "-->":
                         pos += 3
                         state = State_Text
                         break
@@ -496,7 +496,7 @@ class Highlighter(QSyntaxHighlighter):
                 while pos < len_:
                     ch = text[pos]
                     pos += 1
-                    if ch == u'>':
+                    if ch == '>':
                         state = State_Text
                         break
                 self.setFormat(start, pos - start, self.colors['doctype'])
@@ -507,7 +507,7 @@ class Highlighter(QSyntaxHighlighter):
                 while pos < len_:
                     ch = text[pos]
                     pos += 1
-                    if ch == u'>':
+                    if ch == '>':
                         state = State_Text
                         break
                     if not ch.isspace():
@@ -525,7 +525,7 @@ class Highlighter(QSyntaxHighlighter):
                         pos -= 1
                         state = State_InsideTag
                         break
-                    if ch == u'>':
+                    if ch == '>':
                         state = State_Text
                         break
                 self.setFormat(start, pos - start, self.colors['tag'])
@@ -538,10 +538,10 @@ class Highlighter(QSyntaxHighlighter):
                     ch = text[pos]
                     pos += 1
 
-                    if ch == u'/':
+                    if ch == '/':
                         continue
 
-                    if ch == u'>':
+                    if ch == '>':
                         state = State_Text
                         break
 
@@ -558,11 +558,11 @@ class Highlighter(QSyntaxHighlighter):
                     ch = text[pos]
                     pos += 1
 
-                    if ch == u'=':
+                    if ch == '=':
                         state = State_AttributeValue
                         break
 
-                    if ch in (u'>', u'/'):
+                    if ch in ('>', '/'):
                         state = State_InsideTag
                         break
 
@@ -578,12 +578,12 @@ class Highlighter(QSyntaxHighlighter):
                     pos += 1
 
                     # handle opening single quote
-                    if ch == u"'":
+                    if ch == "'":
                         state = State_SingleQuote
                         break
 
                     # handle opening double quote
-                    if ch == u'"':
+                    if ch == '"':
                         state = State_DoubleQuote
                         break
 
@@ -598,7 +598,7 @@ class Highlighter(QSyntaxHighlighter):
                         ch = text[pos]
                         if ch.isspace():
                             break
-                        if ch in (u'>', u'/'):
+                        if ch in ('>', '/'):
                             break
                         pos += 1
                     state = State_InsideTag
@@ -611,7 +611,7 @@ class Highlighter(QSyntaxHighlighter):
                 while pos < len_:
                     ch = text[pos]
                     pos += 1
-                    if ch == u"'":
+                    if ch == "'":
                         break
 
                 state = State_InsideTag
@@ -625,7 +625,7 @@ class Highlighter(QSyntaxHighlighter):
                 while pos < len_:
                     ch = text[pos]
                     pos += 1
-                    if ch == u'"':
+                    if ch == '"':
                         break
 
                 state = State_InsideTag
@@ -636,18 +636,18 @@ class Highlighter(QSyntaxHighlighter):
                 # State_Text and default
                 while pos < len_:
                     ch = text[pos]
-                    if ch == u'<':
-                        if text[pos:pos+4] == u"<!--":
+                    if ch == '<':
+                        if text[pos:pos+4] == "<!--":
                             state = State_Comment
                         else:
-                            if text[pos:pos+9].upper() == u"<!DOCTYPE":
+                            if text[pos:pos+9].upper() == "<!DOCTYPE":
                                 state = State_DocType
                             else:
                                 state = State_TagStart
                         break
-                    elif ch == u'&':
+                    elif ch == '&':
                         start = pos
-                        while pos < len_ and text[pos] != u';':
+                        while pos < len_ and text[pos] != ';':
                             self.setFormat(start, pos - start,
                                     self.colors['entity'])
                             pos += 1
@@ -762,15 +762,14 @@ class Editor(QWidget):  # {{{
     def set_minimum_height_for_editor(self, val):
         self.editor.setMinimumHeight(val)
 
-    @dynamic_property
+    @property
     def html(self):
-        def fset(self, v):
-            self.editor.html = v
+        self.tabs.setCurrentIndex(0)
+        return self.editor.html
 
-        def fget(self):
-            self.tabs.setCurrentIndex(0)
-            return self.editor.html
-        return property(fget=fget, fset=fset)
+    @html.setter
+    def html(self, v):
+        self.editor.html = v
 
     def change_tab(self, index):
         # print 'reloading:', (index and self.wyswyg_dirty) or (not index and
@@ -781,17 +780,16 @@ class Editor(QWidget):  # {{{
                 self.wyswyg_dirty = False
         elif index == 0:  # changing to wyswyg
             if self.source_dirty:
-                self.editor.html = unicode(self.code_edit.toPlainText())
+                self.editor.html = unicode_type(self.code_edit.toPlainText())
                 self.source_dirty = False
 
-    @dynamic_property
+    @property
     def tab(self):
-        def fget(self):
-            return 'code' if self.tabs.currentWidget() is self.code_edit else 'wyswyg'
+        return 'code' if self.tabs.currentWidget() is self.code_edit else 'wyswyg'
 
-        def fset(self, val):
-            self.tabs.setCurrentWidget(self.code_edit if val == 'code' else self.wyswyg)
-        return property(fget=fget, fset=fset)
+    @tab.setter
+    def tab(self, val):
+        self.tabs.setCurrentWidget(self.code_edit if val == 'code' else self.wyswyg)
 
     def wyswyg_dirtied(self, *args):
         self.wyswyg_dirty = True
@@ -815,14 +813,13 @@ class Editor(QWidget):  # {{{
         if self.toolbar_prefs_name is not None:
             gprefs.set(self.toolbar_prefs_name, visible)
 
-    @dynamic_property
+    @property
     def toolbars_visible(self):
-        def fget(self):
-            return self.toolbar1.isVisible() or self.toolbar2.isVisible() or self.toolbar3.isVisible()
+        return self.toolbar1.isVisible() or self.toolbar2.isVisible() or self.toolbar3.isVisible()
 
-        def fset(self, val):
-            getattr(self, ('show' if val else 'hide') + '_toolbars')()
-        return property(fget=fget, fset=fset)
+    @toolbars_visible.setter
+    def toolbars_visible(self, val):
+        getattr(self, ('show' if val else 'hide') + '_toolbars')()
 
     def set_readonly(self, what):
         self.editor.set_readonly(what)
