@@ -1,18 +1,20 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+import os
 from functools import partial
 
 from PyQt5.Qt import (QComboBox, QLabel, QSpinBox, QDoubleSpinBox, QDateTimeEdit,
-        QDateTime, QGroupBox, QVBoxLayout, QSizePolicy, QGridLayout,
+        QDateTime, QGroupBox, QVBoxLayout, QSizePolicy, QGridLayout, QUrl,
         QSpacerItem, QIcon, QCheckBox, QWidget, QHBoxLayout, QLineEdit,
         QPushButton, QMessageBox, QToolButton, Qt, QPlainTextEdit)
 
-from calibre.utils.date import qt_to_dt, now, as_local_time, as_utc
+from calibre.utils.date import qt_to_dt, now, as_local_time, as_utc, internal_iso_format_string
 from calibre.gui2.complete2 import EditWithComplete
 from calibre.gui2.comments_editor import Editor as CommentsEditor
 from calibre.gui2 import UNDEFINED_QDATETIME, error_dialog
@@ -22,6 +24,14 @@ from calibre.utils.icu import sort_key
 from calibre.library.comments import comments_to_html
 from calibre.gui2.library.delegates import ClearingDoubleSpinBox, ClearingSpinBox
 from calibre.gui2.widgets2 import RatingEditor
+from polyglot.builtins import unicode_type
+
+
+def safe_disconnect(signal):
+    try:
+        signal.disconnect()
+    except Exception:
+        pass
 
 
 class Base(object):
@@ -30,6 +40,7 @@ class Base(object):
         self.db, self.col_id = db, col_id
         self.col_metadata = db.custom_column_num_map[col_id]
         self.initial_val = self.widgets = None
+        self.signals_to_disconnect = []
         self.setup_ui(parent)
 
     def initialize(self, book_id):
@@ -65,6 +76,12 @@ class Base(object):
 
     def break_cycles(self):
         self.db = self.widgets = self.initial_val = None
+        for signal in self.signals_to_disconnect:
+            safe_disconnect(signal)
+        self.signals_to_disconnect = []
+
+    def connect_data_changed(self, slot):
+        pass
 
 
 class SimpleText(Base):
@@ -73,11 +90,15 @@ class SimpleText(Base):
         self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), QLineEdit(parent)]
 
     def setter(self, val):
-        self.widgets[1].setText(type(u'')(val or ''))
+        self.widgets[1].setText(unicode_type(val or ''))
 
     def getter(self):
         return self.widgets[1].text().strip()
 
+    def connect_data_changed(self, slot):
+        self.widgets[1].textChanged.connect(slot)
+        self.signals_to_disconnect.append(self.widgets[1].textChanged)
+
 
 class LongText(Base):
 
@@ -92,11 +113,15 @@ class LongText(Base):
         self.widgets = [self._box]
 
     def setter(self, val):
-        self._tb.setPlainText(type(u'')(val or ''))
+        self._tb.setPlainText(unicode_type(val or ''))
 
     def getter(self):
         return self._tb.toPlainText()
 
+    def connect_data_changed(self, slot):
+        self._tb.textChanged.connect(slot)
+        self.signals_to_disconnect.append(self._tb.textChanged)
+
 
 class Bool(Base):
 
@@ -164,6 +189,10 @@ class Bool(Base):
     def set_to_cleared(self):
         self.combobox.setCurrentIndex(2)
 
+    def connect_data_changed(self, slot):
+        self.combobox.currentTextChanged.connect(slot)
+        self.signals_to_disconnect.append(self.combobox.currentTextChanged)
+
 
 class Int(Base):
 
@@ -194,6 +223,10 @@ class Int(Base):
             self.setter(0)
         self.was_none = to_what == self.widgets[1].minimum()
 
+    def connect_data_changed(self, slot):
+        self.widgets[1].valueChanged.connect(slot)
+        self.signals_to_disconnect.append(self.widgets[1].valueChanged)
+
 
 class Float(Int):
 
@@ -222,6 +255,10 @@ class Rating(Base):
     def getter(self):
         return self.widgets[1].rating_value or None
 
+    def connect_data_changed(self, slot):
+        self.widgets[1].currentTextChanged.connect(slot)
+        self.signals_to_disconnect.append(self.widgets[1].currentTextChanged)
+
 
 class DateTimeEdit(QDateTimeEdit):
 
@@ -273,6 +310,8 @@ class DateTime(Base):
         format_ = cm['display'].get('date_format','')
         if not format_:
             format_ = 'dd MMM yyyy hh:mm'
+        elif format_ == 'iso':
+            format_ = internal_iso_format_string()
         w.setDisplayFormat(format_)
         w.setCalendarPopup(True)
         w.setMinimumDateTime(UNDEFINED_QDATETIME)
@@ -301,6 +340,10 @@ class DateTime(Base):
     def normalize_ui_val(self, val):
         return as_utc(val) if val is not None else None
 
+    def connect_data_changed(self, slot):
+        self.widgets[1].dateTimeChanged.connect(slot)
+        self.signals_to_disconnect.append(self.widgets[1].dateTimeChanged)
+
 
 class Comments(Base):
 
@@ -308,13 +351,19 @@ class Comments(Base):
         self._box = QGroupBox(parent)
         self._box.setTitle('&'+self.col_metadata['name'])
         self._layout = QVBoxLayout()
-        self._tb = CommentsEditor(self._box, toolbar_prefs_name=u'metadata-comments-editor-widget-hidden-toolbars')
+        self._tb = CommentsEditor(self._box, toolbar_prefs_name='metadata-comments-editor-widget-hidden-toolbars')
         self._tb.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
         # self._tb.setTabChangesFocus(True)
         self._layout.addWidget(self._tb)
         self._box.setLayout(self._layout)
         self.widgets = [self._box]
 
+    def initialize(self, book_id):
+        path = self.db.abspath(book_id, index_is_id=True)
+        if path:
+            self._tb.set_base_url(QUrl.fromLocalFile(os.path.join(path, 'metadata.html')))
+        return Base.initialize(self, book_id)
+
     def setter(self, val):
         if not val or not val.strip():
             val = ''
@@ -324,19 +373,22 @@ class Comments(Base):
         self._tb.wyswyg_dirtied()
 
     def getter(self):
-        val = unicode(self._tb.html).strip()
+        val = unicode_type(self._tb.html).strip()
         if not val:
             val = None
         return val
 
-    @dynamic_property
+    @property
     def tab(self):
-        def fget(self):
-            return self._tb.tab
+        return self._tb.tab
 
-        def fset(self, val):
-            self._tb.tab = val
-        return property(fget=fget, fset=fset)
+    @tab.setter
+    def tab(self, val):
+        self._tb.tab = val
+
+    def connect_data_changed(self, slot):
+        self._tb.data_changed.connect(slot)
+        self.signals_to_disconnect.append(self._tb.data_changed)
 
 
 class MultipleWidget(QWidget):
@@ -389,6 +441,14 @@ class MultipleWidget(QWidget):
         return self.tags_box.text()
 
 
+def _save_dialog(parent, title, msg, det_msg=''):
+    d = QMessageBox(parent)
+    d.setWindowTitle(title)
+    d.setText(msg)
+    d.setStandardButtons(QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
+    return d.exec_()
+
+
 class Text(Base):
 
     def setup_ui(self, parent):
@@ -438,27 +498,19 @@ class Text(Base):
 
     def getter(self):
         if self.col_metadata['is_multiple']:
-            val = unicode(self.widgets[1].text()).strip()
+            val = unicode_type(self.widgets[1].text()).strip()
             ans = [x.strip() for x in val.split(self.sep['ui_to_list']) if x.strip()]
             if not ans:
                 ans = None
             return ans
-        val = unicode(self.widgets[1].currentText()).strip()
+        val = unicode_type(self.widgets[1].currentText()).strip()
         if not val:
             val = None
         return val
 
-    def _save_dialog(self, parent, title, msg, det_msg=''):
-        d = QMessageBox(parent)
-        d.setWindowTitle(title)
-        d.setText(msg)
-        d.setStandardButtons(QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
-        return d.exec_()
-
     def edit(self):
-        if (self.getter() != self.initial_val and (self.getter() or
-            self.initial_val)):
-            d = self._save_dialog(self.parent, _('Values changed'),
+        if (self.getter() != self.initial_val and (self.getter() or self.initial_val)):
+            d = _save_dialog(self.parent, _('Values changed'),
                     _('You have changed the values. In order to use this '
                        'editor, you must either discard or apply these '
                        'changes. Apply changes?'))
@@ -474,6 +526,14 @@ class Text(Base):
         if d.exec_() == TagEditor.Accepted:
             self.setter(d.tags)
 
+    def connect_data_changed(self, slot):
+        if self.col_metadata['is_multiple']:
+            s = self.widgets[1].tags_box.currentTextChanged
+        else:
+            s = self.widgets[1].currentTextChanged
+        s.connect(slot)
+        self.signals_to_disconnect.append(s)
+
 
 class Series(Base):
 
@@ -512,7 +572,7 @@ class Series(Base):
         self.initial_val, self.initial_index = self.current_val
 
     def getter(self):
-        n = unicode(self.name_widget.currentText()).strip()
+        n = unicode_type(self.name_widget.currentText()).strip()
         i = self.idx_widget.value()
         return n, i
 
@@ -547,6 +607,11 @@ class Series(Base):
         val, s_index = self.current_val
         mi.set('#' + self.col_metadata['label'], val, extra=s_index)
 
+    def connect_data_changed(self, slot):
+        for s in self.widgets[1].editTextChanged, self.widgets[3].valueChanged:
+            s.connect(slot)
+            self.signals_to_disconnect.append(s)
+
 
 class Enumeration(Base):
 
@@ -579,7 +644,7 @@ class Enumeration(Base):
         self.widgets[1].setCurrentIndex(self.widgets[1].findText(val))
 
     def getter(self):
-        return unicode(self.widgets[1].currentText())
+        return unicode_type(self.widgets[1].currentText())
 
     def normalize_db_val(self, val):
         if val is None:
@@ -591,6 +656,10 @@ class Enumeration(Base):
             val = None
         return val
 
+    def connect_data_changed(self, slot):
+        self.widgets[1].currentIndexChanged.connect(slot)
+        self.signals_to_disconnect.append(self.widgets[1].currentIndexChanged)
+
 
 def comments_factory(db, key, parent):
     fm = db.custom_column_num_map[key]
@@ -662,13 +731,14 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
     count = len(cols)
     layout_rows_for_comments = 9
     if two_column:
-        turnover_point = ((count-comments_not_in_tweak+1) +
-                          comments_in_tweak*(layout_rows_for_comments-1))/2
+        turnover_point = int(((count - comments_not_in_tweak + 1) +
+                                int(comments_in_tweak*(layout_rows_for_comments-1)))/2)
     else:
         # Avoid problems with multi-line widgets
         turnover_point = count + 1000
     ans = []
     column = row = base_row = max_row = 0
+    minimum_label = 0
     for key in cols:
         if not fm[key]['is_editable']:
             continue  # this almost never happens
@@ -689,7 +759,7 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
                 column = 0
                 row = max_row
                 base_row = row
-                turnover_point = row + (comments_not_in_tweak * layout_rows_for_comments)/2
+                turnover_point = row + int((comments_not_in_tweak * layout_rows_for_comments)/2)
                 comments_not_in_tweak = 0
 
         l = QGridLayout()
@@ -704,10 +774,33 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
         for c in range(0, len(w.widgets), 2):
             if not is_comments:
                 w.widgets[c].setWordWrap(True)
+                '''
+                It seems that there is something strange with wordwrapped labels
+                with some fonts. Apparently one part of QT thinks it is showing
+                a single line and sizes the line vertically accordingly. Another
+                part thinks there isn't enough space and wraps the label. The
+                result is two lines in a single line space, cutting off parts of
+                the lines. It doesn't happen with every font, nor with every
+                "long" label.
+
+                This change works around the problem by setting the maximum
+                display width and telling QT to respect that width.
+
+                While here I implemented an arbitrary minimum label length so
+                that there is a better chance that the field edit boxes line up.
+                '''
+                if minimum_label == 0:
+                    minimum_label = w.widgets[c].fontMetrics().boundingRect('smallLabel').width()
+                label_width = w.widgets[c].fontMetrics().boundingRect(w.widgets[c].text()).width()
+                if c == 0:
+                    w.widgets[0].setMaximumWidth(label_width)
+                    w.widgets[0].setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
+                    l.setColumnMinimumWidth(0, minimum_label)
+                else:
+                    w.widgets[0].setMaximumWidth(max(w.widgets[0].maximumWidth(), label_width))
                 w.widgets[c].setBuddy(w.widgets[c+1])
                 l.addWidget(w.widgets[c], c, 0)
                 l.addWidget(w.widgets[c+1], c, 1)
-                l.setColumnStretch(1, 10000)
             else:
                 l.addWidget(w.widgets[0], 0, 0, 1, 2)
         l.addItem(QSpacerItem(0, 0, vPolicy=QSizePolicy.Expanding), c, 0, 1, 1)
@@ -735,7 +828,7 @@ class BulkBase(Base):
         return self._cached_gui_val_
 
     def get_initial_value(self, book_ids):
-        values = set([])
+        values = set()
         for book_id in book_ids:
             val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
             if isinstance(val, list):
@@ -745,7 +838,7 @@ class BulkBase(Base):
                 break
         ans = None
         if len(values) == 1:
-            ans = iter(values).next()
+            ans = next(iter(values))
         if isinstance(ans, frozenset):
             ans = list(ans)
         return ans
@@ -762,7 +855,8 @@ class BulkBase(Base):
         val = self.normalize_ui_val(val)
         self.db.set_custom_bulk(book_ids, val, num=self.col_id, notify=notify)
 
-    def make_widgets(self, parent, main_widget_class, extra_label_text=''):
+    def make_widgets(self, parent, main_widget_class, extra_label_text='',
+                     add_tags_edit_button=False):
         w = QWidget(parent)
         self.widgets = [QLabel('&'+self.col_metadata['name']+':', w), w]
         l = QHBoxLayout()
@@ -771,6 +865,11 @@ class BulkBase(Base):
         self.main_widget = main_widget_class(w)
         l.addWidget(self.main_widget)
         l.setStretchFactor(self.main_widget, 10)
+        if add_tags_edit_button:
+            self.edit_tags_button = QToolButton(parent)
+            self.edit_tags_button.setToolTip(_('Open Item Editor'))
+            self.edit_tags_button.setIcon(QIcon(I('chapters.png')))
+            l.addWidget(self.edit_tags_button)
         self.a_c_checkbox = QCheckBox(_('Apply changes'), w)
         l.addWidget(self.a_c_checkbox)
         self.ignore_change_signals = True
@@ -929,10 +1028,12 @@ class BulkDateTime(BulkBase):
         l.addStretch(2)
 
         w = self.main_widget
-        format = cm['display'].get('date_format','')
-        if not format:
-            format = 'dd MMM yyyy'
-        w.setDisplayFormat(format)
+        format_ = cm['display'].get('date_format','')
+        if not format_:
+            format_ = 'dd MMM yyyy'
+        elif format_ == 'iso':
+            format_ = internal_iso_format_string()
+        w.setDisplayFormat(format_)
         w.setCalendarPopup(True)
         w.setMinimumDateTime(UNDEFINED_QDATETIME)
         w.setSpecialValueText(_('Undefined'))
@@ -979,20 +1080,20 @@ class BulkSeries(BulkBase):
         layout.addWidget(self.remove_series)
         self.idx_widget = QCheckBox(parent)
         self.idx_widget.setText(_('Automatically number books'))
-        self.idx_widget.setToolTip('<p>' +
-                       _('If not checked, the series number for the books will be set to 1. '
-                         'If checked, selected books will be automatically numbered, '
-                         'in the order you selected them. So if you selected '
-                         'Book A and then Book B, Book A will have series number 1 '
-                         'and Book B series number 2.') + '</p>')
+        self.idx_widget.setToolTip('<p>' + _(
+            'If not checked, the series number for the books will be set to 1. '
+            'If checked, selected books will be automatically numbered, '
+            'in the order you selected them. So if you selected '
+            'Book A and then Book B, Book A will have series number 1 '
+            'and Book B series number 2.') + '</p>')
         layout.addWidget(self.idx_widget)
         self.force_number = QCheckBox(parent)
         self.force_number.setText(_('Force numbers to start with '))
-        self.force_number.setToolTip('<p>' +
-                         _('Series will normally be renumbered from the highest '
-                           'number in the database for that series. Checking this '
-                           'box will tell calibre to start numbering from the value '
-                           'in the box') + '</p>')
+        self.force_number.setToolTip('<p>' + _(
+            'Series will normally be renumbered from the highest '
+            'number in the database for that series. Checking this '
+            'box will tell calibre to start numbering from the value '
+            'in the box') + '</p>')
         layout.addWidget(self.force_number)
         self.series_start_number = QDoubleSpinBox(parent)
         self.series_start_number.setMinimum(0.0)
@@ -1003,10 +1104,10 @@ class BulkSeries(BulkBase):
         self.series_increment.setMinimum(0.00)
         self.series_increment.setMaximum(99999.0)
         self.series_increment.setProperty("value", 1.0)
-        self.series_increment.setToolTip('<p>' +
-                         _('The amount by which to increment the series number '
-                           'for successive books. Only applicable when using '
-                           'force series numbers.') + '</p>')
+        self.series_increment.setToolTip('<p>' + _(
+            'The amount by which to increment the series number '
+            'for successive books. Only applicable when using '
+            'force series numbers.') + '</p>')
         self.series_increment.setPrefix('+')
         layout.addWidget(self.series_increment)
         layout.addItem(QSpacerItem(20, 10, QSizePolicy.Expanding, QSizePolicy.Minimum))
@@ -1065,7 +1166,7 @@ class BulkSeries(BulkBase):
         self.a_c_checkbox.setChecked(False)
 
     def getter(self):
-        n = unicode(self.main_widget.currentText()).strip()
+        n = unicode_type(self.main_widget.currentText()).strip()
         autonumber = self.idx_widget.checkState()
         force = self.force_number.checkState()
         start = self.series_start_number.value()
@@ -1135,7 +1236,7 @@ class BulkEnumeration(BulkBase, Enumeration):
         self.main_widget.blockSignals(False)
 
     def getter(self):
-        return unicode(self.main_widget.currentText())
+        return unicode_type(self.main_widget.currentText())
 
     def setter(self, val):
         if val is None:
@@ -1156,6 +1257,10 @@ class RemoveTags(QWidget):
         self.tags_box = EditWithComplete(parent)
         self.tags_box.update_items_cache(values)
         layout.addWidget(self.tags_box, stretch=3)
+        self.remove_tags_button = QToolButton(parent)
+        self.remove_tags_button.setToolTip(_('Open Item Editor'))
+        self.remove_tags_button.setIcon(QIcon(I('chapters.png')))
+        layout.addWidget(self.remove_tags_button)
         self.checkbox = QCheckBox(_('Remove all tags'), parent)
         layout.addWidget(self.checkbox)
         layout.addStretch(1)
@@ -1176,15 +1281,18 @@ class BulkText(BulkBase):
         values = self.all_values = list(self.db.all_custom(num=self.col_id))
         values.sort(key=sort_key)
         if self.col_metadata['is_multiple']:
+            is_tags = not self.col_metadata['display'].get('is_names', False)
             self.make_widgets(parent, EditWithComplete,
-                              extra_label_text=_('tags to add'))
+                              extra_label_text=_('tags to add'),
+                              add_tags_edit_button=is_tags)
             self.main_widget.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
             self.adding_widget = self.main_widget
 
-            if not self.col_metadata['display'].get('is_names', False):
+            if is_tags:
+                self.edit_tags_button.clicked.connect(self.edit_add)
                 w = RemoveTags(parent, values)
-                self.widgets.append(QLabel('&'+self.col_metadata['name']+': ' +
-                                           _('tags to remove'), parent))
+                w.remove_tags_button.clicked.connect(self.edit_remove)
+                self.widgets.append(QLabel('&'+self.col_metadata['name']+': ' + _('tags to remove'), parent))
                 self.widgets.append(w)
                 self.removing_widget = w
                 self.main_widget.set_separator(',')
@@ -1202,6 +1310,7 @@ class BulkText(BulkBase):
                         self.main_widget.AdjustToMinimumContentsLengthWithIcon)
             self.main_widget.setMinimumContentsLength(25)
         self.ignore_change_signals = False
+        self.parent = parent
 
     def initialize(self, book_ids):
         self.main_widget.update_items_cache(self.all_values)
@@ -1231,10 +1340,10 @@ class BulkText(BulkBase):
                 else:
                     txt = rtext
                     if txt:
-                        remove = set([v.strip() for v in txt.split(ism['ui_to_list'])])
+                        remove = {v.strip() for v in txt.split(ism['ui_to_list'])}
                 txt = adding
                 if txt:
-                    add = set([v.strip() for v in txt.split(ism['ui_to_list'])])
+                    add = {v.strip() for v in txt.split(ism['ui_to_list'])}
                 else:
                     add = set()
                 self.db.set_custom_bulk_multiple(book_ids, add=add,
@@ -1248,14 +1357,36 @@ class BulkText(BulkBase):
         if self.col_metadata['is_multiple']:
             if not self.col_metadata['display'].get('is_names', False):
                 return self.removing_widget.checkbox.isChecked(), \
-                        unicode(self.adding_widget.text()), \
-                        unicode(self.removing_widget.tags_box.text())
-            return unicode(self.adding_widget.text())
-        val = unicode(self.main_widget.currentText()).strip()
+                        unicode_type(self.adding_widget.text()), \
+                        unicode_type(self.removing_widget.tags_box.text())
+            return unicode_type(self.adding_widget.text())
+        val = unicode_type(self.main_widget.currentText()).strip()
         if not val:
             val = None
         return val
 
+    def edit_remove(self):
+        self.edit(widget=self.removing_widget.tags_box)
+
+    def edit_add(self):
+        self.edit(widget=self.main_widget)
+
+    def edit(self, widget):
+        if widget.text():
+            d = _save_dialog(self.parent, _('Values changed'),
+                    _('You have entered values. In order to use this '
+                       'editor you must first discard them. '
+                       'Discard the values?'))
+            if d == QMessageBox.Cancel or d == QMessageBox.No:
+                return
+            widget.setText('')
+        d = TagEditor(self.parent, self.db, key=('#'+self.col_metadata['label']))
+        if d.exec_() == TagEditor.Accepted:
+            val = d.tags
+            if not val:
+                val = []
+            widget.setText(self.col_metadata['multiple_seps']['list_to_ui'].join(val))
+
 
 bulk_widgets = {
         'bool' : BulkBool,
