@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -16,6 +15,7 @@ from PyQt5.Qt import (
 from calibre.utils.dbus_service import Object, BusName, method as dbus_method, dbus_property, signal as dbus_signal
 from calibre.gui2.dbus_export.utils import (
     setup_for_cli_run, swap_mnemonic_char, key_sequence_to_dbus_shortcut, icon_to_dbus_menu_icon)
+from polyglot.builtins import iteritems
 
 null = object()
 
@@ -124,13 +124,13 @@ class DBusMenu(QObject):
             return {}
         ans = self._action_properties.get(action_id, PropDict())
         if restrict_to:
-            ans = PropDict({k:v for k, v in ans.iteritems() if k in restrict_to})
+            ans = PropDict({k:v for k, v in iteritems(ans) if k in restrict_to})
         return ans
 
     def publish_new_menu(self, qmenu=None):
         self.init_maps(qmenu)
         if qmenu is not None:
-            qmenu.destroyed.connect(lambda obj=None:self.publish_new_menu())
+            connect_lambda(qmenu.destroyed, self, lambda self:self.publish_new_menu())
             ac = qmenu.menuAction()
             self.add_action(ac)
         self.dbus_api.LayoutUpdated(self.dbus_api.revision, 0)
@@ -195,7 +195,7 @@ class DBusMenu(QObject):
             removed = set(old_props) - set(new_props)
             if removed:
                 removed_props.append((ac_id, dbus.Array(removed, signature='as')))
-            updated = PropDict({k:v for k, v in new_props.iteritems() if v != old_props.get(k, null)})
+            updated = PropDict({k:v for k, v in iteritems(new_props) if v != old_props.get(k, null)})
             if updated:
                 updated_props.append((ac_id, updated))
         self.action_changes = set()
@@ -237,7 +237,7 @@ class DBusMenu(QObject):
         return parent_id, props, self.get_layout_children(parent_id, depth, property_names)
 
     def get_layout_children(self, parent_id, depth, property_names):
-        ans = dbus.Array(signature='(ia{sv}av)')
+        ans = dbus.Array(signature='v')
         ac = self.id_to_action(parent_id)
         if ac is not None and depth != 0 and ac.menu() is not None:
             for child in menu_actions(ac.menu()):
@@ -388,5 +388,6 @@ def test():
     app.exec_()
     del dbus_name
 
+
 if __name__ == '__main__':
     test()
