@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -10,11 +9,14 @@ __docformat__ = 'restructuredtext en'
 from collections import OrderedDict
 from functools import partial
 
-import sip
 from PyQt5.Qt import (QObject, QKeySequence, QAbstractItemModel, QModelIndex,
         Qt, QStyledItemDelegate, QTextDocument, QStyle, pyqtSignal, QFrame,
         QApplication, QSize, QRectF, QWidget, QTreeView,
         QGridLayout, QLabel, QRadioButton, QPushButton, QToolButton, QIcon)
+try:
+    from PyQt5 import sip
+except ImportError:
+    import sip
 
 from calibre.utils.config import JSONConfig
 from calibre.constants import DEBUG
@@ -23,12 +25,15 @@ from calibre.utils.icu import sort_key, lower
 from calibre.gui2 import error_dialog, info_dialog
 from calibre.utils.search_query_parser import SearchQueryParser, ParseException
 from calibre.gui2.search_box import SearchBox2
+from polyglot.builtins import iteritems, itervalues, unicode_type, range
 
 ROOT = QModelIndex()
 
+
 class NameConflict(ValueError):
     pass
 
+
 def keysequence_from_event(ev):  # {{{
     k, mods = ev.key(), int(ev.modifiers())
     if k in (
@@ -44,6 +49,7 @@ def keysequence_from_event(ev):  # {{{
     return QKeySequence(k | mods)
 # }}}
 
+
 def finalize(shortcuts, custom_keys_map={}):  # {{{
     '''
     Resolve conflicts and assign keys to every action in shortcuts, which must
@@ -53,7 +59,7 @@ def finalize(shortcuts, custom_keys_map={}):  # {{{
     correctly for each shortcut.
     '''
     seen, keys_map = {}, {}
-    for unique_name, shortcut in shortcuts.iteritems():
+    for unique_name, shortcut in iteritems(shortcuts):
         custom_keys = custom_keys_map.get(unique_name, None)
         if custom_keys is None:
             candidates = shortcut['default_keys']
@@ -64,7 +70,7 @@ def finalize(shortcuts, custom_keys_map={}):  # {{{
         keys = []
         for x in candidates:
             ks = QKeySequence(x, QKeySequence.PortableText)
-            x = unicode(ks.toString(QKeySequence.PortableText))
+            x = unicode_type(ks.toString(QKeySequence.PortableText))
             if x in seen:
                 if DEBUG:
                     prints('Key %r for shortcut %s is already used by'
@@ -87,6 +93,7 @@ def finalize(shortcuts, custom_keys_map={}):  # {{{
 
 # }}}
 
+
 class Manager(QObject):  # {{{
 
     def __init__(self, parent=None, config_name='shortcuts/main'):
@@ -134,15 +141,15 @@ class Manager(QObject):  # {{{
         done unregistering.
         '''
         self.shortcuts.pop(unique_name, None)
-        for group in self.groups.itervalues():
+        for group in itervalues(self.groups):
             try:
                 group.remove(unique_name)
             except ValueError:
                 pass
 
     def finalize(self):
-        custom_keys_map = {un:tuple(keys) for un, keys in self.config.get(
-            'map', {}).iteritems()}
+        custom_keys_map = {un:tuple(keys) for un, keys in iteritems(self.config.get(
+            'map', {}))}
         self.keys_map = finalize(self.shortcuts, custom_keys_map=custom_keys_map)
 
     def replace_action(self, unique_name, new_action):
@@ -159,6 +166,7 @@ class Manager(QObject):  # {{{
 
 # Model {{{
 
+
 class Node(object):
 
     def __init__(self, group_map, shortcut_map, name=None, shortcut=None):
@@ -179,6 +187,7 @@ class Node(object):
         for child in self.children:
             yield child
 
+
 class ConfigModel(SearchQueryParser, QAbstractItemModel):
 
     def __init__(self, keyboard, parent=None):
@@ -188,16 +197,16 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
         self.keyboard = keyboard
         groups = sorted(keyboard.groups, key=sort_key)
         shortcut_map = {k:v.copy() for k, v in
-                self.keyboard.shortcuts.iteritems()}
-        for un, s in shortcut_map.iteritems():
+                iteritems(self.keyboard.shortcuts)}
+        for un, s in iteritems(shortcut_map):
             s['keys'] = tuple(self.keyboard.keys_map.get(un, ()))
             s['unique_name'] = un
-            s['group'] = [g for g, names in self.keyboard.groups.iteritems() if un in
+            s['group'] = [g for g, names in iteritems(self.keyboard.groups) if un in
                     names][0]
 
         group_map = {group:sorted(names, key=lambda x:
                 sort_key(shortcut_map[x]['name'])) for group, names in
-                self.keyboard.groups.iteritems()}
+                iteritems(self.keyboard.groups)}
 
         self.data = [Node(group_map, shortcut_map, group) for group in groups]
 
@@ -260,7 +269,7 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
         for node in self.all_shortcuts:
             s = node.data
             s['keys'] = tuple(keys_map[s['unique_name']])
-        for r in xrange(self.rowCount()):
+        for r in range(self.rowCount()):
             group = self.index(r, 0)
             num = self.rowCount(group)
             if num > 0:
@@ -273,7 +282,7 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
             sc = node.data
             if sc['set_to_default']:
                 continue
-            keys = [unicode(k.toString(k.PortableText)) for k in sc['keys']]
+            keys = [unicode_type(k.toString(k.PortableText)) for k in sc['keys']]
             kmap[sc['unique_name']] = keys
         self.keyboard.config['map'] = kmap
 
@@ -288,7 +297,7 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
     def get_matches(self, location, query, candidates=None):
         if candidates is None:
             candidates = self.universal_set()
-        ans = set([])
+        ans = set()
         if not query:
             return ans
         query = lower(query)
@@ -360,6 +369,7 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
 
 # }}}
 
+
 class Editor(QFrame):  # {{{
 
     editing_done = pyqtSignal(object)
@@ -418,11 +428,11 @@ class Editor(QFrame):  # {{{
         self.default_keys = [QKeySequence(k, QKeySequence.PortableText) for k
                 in shortcut['default_keys']]
         self.current_keys = list(shortcut['keys'])
-        default = ', '.join([unicode(k.toString(k.NativeText)) for k in
+        default = ', '.join([unicode_type(k.toString(k.NativeText)) for k in
                     self.default_keys])
         if not default:
             default = _('None')
-        current = ', '.join([unicode(k.toString(k.NativeText)) for k in
+        current = ', '.join([unicode_type(k.toString(k.NativeText)) for k in
                     self.current_keys])
         if not current:
             current = _('None')
@@ -480,8 +490,8 @@ class Editor(QFrame):  # {{{
         dup_desc = self.dup_check(sequence)
         if dup_desc is not None:
             error_dialog(self, _('Already assigned'),
-                    unicode(sequence.toString(QKeySequence.NativeText)) + ' ' +
-                    _('already assigned to') + ' ' + dup_desc, show=True)
+                    unicode_type(sequence.toString(QKeySequence.NativeText)) + ' ' + _(
+                        'already assigned to') + ' ' + dup_desc, show=True)
             self.clear_clicked(which=which)
 
     def dup_check(self, sequence):
@@ -499,7 +509,7 @@ class Editor(QFrame):  # {{{
         ans = []
         for which in (1, 2):
             button = getattr(self, 'button%d'%which)
-            t = unicode(button.text())
+            t = unicode_type(button.text())
             if t == _('None'):
                 continue
             ks = QKeySequence(t, QKeySequence.NativeText)
@@ -526,7 +536,7 @@ class Delegate(QStyledItemDelegate):  # {{{
         elif data.is_shortcut:
             shortcut = data.data
             # Shortcut
-            keys = [unicode(k.toString(k.NativeText)) for k in shortcut['keys']]
+            keys = [unicode_type(k.toString(k.NativeText)) for k in shortcut['keys']]
             if not keys:
                 keys = _('None')
             else:
@@ -614,6 +624,7 @@ class Delegate(QStyledItemDelegate):  # {{{
 
 # }}}
 
+
 class ShortcutConfig(QWidget):  # {{{
 
     changed_signal = pyqtSignal()
@@ -696,7 +707,7 @@ class ShortcutConfig(QWidget):  # {{{
         if not idx.isValid():
             idx = self._model.index(0, 0)
         idx = self._model.find_next(idx,
-                unicode(self.search.currentText()))
+                unicode_type(self.search.currentText()))
         self.highlight_index(idx)
 
     def find_previous(self, *args):
@@ -704,7 +715,7 @@ class ShortcutConfig(QWidget):  # {{{
         if not idx.isValid():
             idx = self._model.index(0, 0)
         idx = self._model.find_next(idx,
-            unicode(self.search.currentText()), backwards=True)
+            unicode_type(self.search.currentText()), backwards=True)
         self.highlight_index(idx)
 
     def highlight_group(self, group_name):
