@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,7 +8,7 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 import os, weakref
 from collections import OrderedDict, namedtuple
 from functools import partial
-from future_builtins import zip
+from polyglot.builtins import iteritems, itervalues, zip, unicode_type, range, map
 
 from PyQt5.Qt import (
     QDialog, QWidget, QGridLayout, QLabel, QToolButton, QIcon,
@@ -49,29 +48,28 @@ class LineEdit(EditWithComplete):
             self.set_separator(sep)
         self.textChanged.connect(self.changed)
 
-    @dynamic_property
+    @property
     def value(self):
-        def fget(self):
-            val = unicode(self.text()).strip()
-            ism = self.metadata['is_multiple']
-            if ism:
-                if not val:
-                    val = []
-                else:
-                    val = val.strip(ism['list_to_ui'].strip())
-                    val = [x.strip() for x in val.split(ism['list_to_ui']) if x.strip()]
-            return val
+        val = unicode_type(self.text()).strip()
+        ism = self.metadata['is_multiple']
+        if ism:
+            if not val:
+                val = []
+            else:
+                val = val.strip(ism['list_to_ui'].strip())
+                val = [x.strip() for x in val.split(ism['list_to_ui']) if x.strip()]
+        return val
 
-        def fset(self, val):
-            ism = self.metadata['is_multiple']
-            if ism:
-                if not val:
-                    val = ''
-                else:
-                    val = ism['list_to_ui'].join(val)
-            self.setText(val)
-            self.setCursorPosition(0)
-        return property(fget=fget, fset=fset)
+    @value.setter
+    def value(self, val):
+        ism = self.metadata['is_multiple']
+        if ism:
+            if not val:
+                val = ''
+            else:
+                val = ism['list_to_ui'].join(val)
+        self.setText(val)
+        self.setCursorPosition(0)
 
     def from_mi(self, mi):
         val = mi.get(self.field, default='') or ''
@@ -85,15 +83,14 @@ class LineEdit(EditWithComplete):
         elif self.field == 'authors':
             mi.set('author_sort', authors_to_sort_string(val))
 
-    @dynamic_property
+    @property
     def current_val(self):
-        def fget(self):
-            return unicode(self.text())
+        return unicode_type(self.text())
 
-        def fset(self, val):
-            self.setText(val)
-            self.setCursorPosition(0)
-        return property(fget=fget, fset=fset)
+    @current_val.setter
+    def current_val(self, val):
+        self.setText(val)
+        self.setCursorPosition(0)
 
     @property
     def is_blank(self):
@@ -119,14 +116,13 @@ class LanguagesEdit(LE):
         if not is_new:
             self.lineEdit().setReadOnly(True)
 
-    @dynamic_property
+    @property
     def current_val(self):
-        def fget(self):
-            return self.lang_codes
+        return self.lang_codes
 
-        def fset(self, val):
-            self.lang_codes = val
-        return property(fget=fget, fset=fset)
+    @current_val.setter
+    def current_val(self, val):
+        self.lang_codes = val
 
     def from_mi(self, mi):
         self.lang_codes = mi.languages
@@ -212,7 +208,7 @@ class SeriesEdit(LineEdit):
         self.setCursorPosition(0)
 
     def to_mi(self, mi):
-        val = unicode(self.text()).strip()
+        val = unicode_type(self.text()).strip()
         try:
             series_index = float(val.rpartition('[')[-1].rstrip(']').strip())
         except:
@@ -241,17 +237,16 @@ class IdentifiersEdit(LineEdit):
     def to_mi(self, mi):
         mi.set_identifiers(self.as_dict)
 
-    @dynamic_property
+    @property
     def as_dict(self):
-        def fget(self):
-            parts = (x.strip() for x in self.current_val.split(',') if x.strip())
-            return {k:v for k, v in {x.partition(':')[0].strip():x.partition(':')[-1].strip() for x in parts}.iteritems() if k and v}
+        parts = (x.strip() for x in self.current_val.split(',') if x.strip())
+        return {k:v for k, v in iteritems({x.partition(':')[0].strip():x.partition(':')[-1].strip() for x in parts}) if k and v}
 
-        def fset(self, val):
-            val = ('%s:%s' % (k, v) for k, v in val.iteritems())
-            self.setText(', '.join(val))
-            self.setCursorPosition(0)
-        return property(fget=fget, fset=fset)
+    @as_dict.setter
+    def as_dict(self, val):
+        val = ('%s:%s' % (k, v) for k, v in iteritems(val))
+        self.setText(', '.join(val))
+        self.setCursorPosition(0)
 
 
 class CommentsEdit(Editor):
@@ -269,15 +264,14 @@ class CommentsEdit(Editor):
             self.hide_toolbars()
             self.set_readonly(True)
 
-    @dynamic_property
+    @property
     def current_val(self):
-        def fget(self):
-            return self.html
+        return self.html
 
-        def fset(self, val):
-            self.html = val or ''
-            self.changed.emit()
-        return property(fget=fget, fset=fset)
+    @current_val.setter
+    def current_val(self, val):
+        self.html = val or ''
+        self.changed.emit()
 
     def from_mi(self, mi):
         val = mi.get(self.field, default='')
@@ -315,16 +309,15 @@ class CoverView(QWidget):
     def is_blank(self):
         return self.pixmap is None
 
-    @dynamic_property
+    @property
     def current_val(self):
-        def fget(self):
-            return self.pixmap
+        return self.pixmap
 
-        def fset(self, val):
-            self.pixmap = val
-            self.changed.emit()
-            self.update()
-        return property(fget=fget, fset=fset)
+    @current_val.setter
+    def current_val(self, val):
+        self.pixmap = val
+        self.changed.emit()
+        self.update()
 
     def from_mi(self, mi):
         p = getattr(mi, 'cover', None)
@@ -431,7 +424,8 @@ class CompareSingle(QWidget):
             else:
                 continue
             neww = cls(field, True, self, m, extra)
-            connect_lambda(neww.changed, self, lambda self: self.changed(field))
+            neww.setObjectName(field)
+            connect_lambda(neww.changed, self, lambda self: self.changed(self.sender().objectName()))
             if isinstance(neww, EditWithComplete):
                 try:
                     neww.update_items_cache(db.new_api.all_field_names(field))
@@ -444,7 +438,8 @@ class CompareSingle(QWidget):
             newl.setBuddy(neww)
             button = RightClickButton(self)
             button.setIcon(QIcon(I('back.png')))
-            connect_lambda(button.clicked, self, lambda self: self.revert(field))
+            button.setObjectName(field)
+            connect_lambda(button.clicked, self, lambda self: self.revert(self.sender().objectName()))
             button.setToolTip(revert_tooltip % m['name'])
             if field == 'identifiers':
                 button.m = m = QMenu(button)
@@ -517,14 +512,14 @@ class CompareSingle(QWidget):
     def __call__(self, oldmi, newmi):
         self.current_mi = newmi
         self.initial_vals = {}
-        for field, widgets in self.widgets.iteritems():
+        for field, widgets in iteritems(self.widgets):
             widgets.old.from_mi(oldmi)
             widgets.new.from_mi(newmi)
             self.initial_vals[field] = widgets.new.current_val
 
     def apply_changes(self):
         changed = False
-        for field, widgets in self.widgets.iteritems():
+        for field, widgets in iteritems(self.widgets):
             val = widgets.new.current_val
             if val != self.initial_vals[field]:
                 widgets.new.to_mi(self.current_mi)
@@ -696,9 +691,9 @@ if __name__ == '__main__':
     ids = sorted(db.all_ids(), reverse=True)
     ids = tuple(zip(ids[0::2], ids[1::2]))
     gm = partial(db.get_metadata, index_is_id=True, get_cover=True, cover_as_data=True)
-    get_metadata = lambda x:map(gm, ids[x])
-    d = CompareMany(list(xrange(len(ids))), get_metadata, db.field_metadata, db=db)
+    get_metadata = lambda x:list(map(gm, ids[x]))
+    d = CompareMany(list(range(len(ids))), get_metadata, db.field_metadata, db=db)
     if d.exec_() == d.Accepted:
-        for changed, mi in d.accepted.itervalues():
+        for changed, mi in itervalues(d.accepted):
             if changed and mi is not None:
-                print (mi)
+                print(mi)
