@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,7 +10,6 @@ import json, textwrap
 
 from collections import defaultdict
 from threading import Thread
-from functools import partial
 
 from PyQt5.Qt import (
     QApplication, QFont, QFontInfo, QFontDialog, QColorDialog, QPainter,
@@ -37,6 +37,7 @@ from calibre.gui2.preferences.coloring import EditRules
 from calibre.gui2.library.alternate_views import auto_height, CM_TO_INCH
 from calibre.gui2.widgets2 import Dialog
 from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+from polyglot.builtins import iteritems, unicode_type, map
 
 
 class BusyCursor(object):
@@ -159,7 +160,7 @@ class IdLinksEditor(Dialog):
         la.setWordWrap(True)
         l.addWidget(la)
         items = []
-        for k, lx in msprefs['id_link_rules'].iteritems():
+        for k, lx in iteritems(msprefs['id_link_rules']):
             for n, t in lx:
                 items.append((k, n, t))
         items.sort(key=lambda x:sort_key(x[1]))
@@ -172,13 +173,13 @@ class IdLinksEditor(Dialog):
         l.addWidget(t)
         t.horizontalHeader().setSectionResizeMode(2, t.horizontalHeader().Stretch)
         self.cb = b = QPushButton(QIcon(I('plus.png')), _('&Add rule'), self)
-        b.clicked.connect(lambda : self.edit_rule())
+        connect_lambda(b.clicked, self, lambda self: self.edit_rule())
         self.bb.addButton(b, self.bb.ActionRole)
         self.rb = b = QPushButton(QIcon(I('minus.png')), _('&Remove rule'), self)
-        b.clicked.connect(lambda : self.remove_rule())
+        connect_lambda(b.clicked, self, lambda self: self.remove_rule())
         self.bb.addButton(b, self.bb.ActionRole)
         self.eb = b = QPushButton(QIcon(I('modified.png')), _('&Edit rule'), self)
-        b.clicked.connect(lambda : self.edit_rule(self.table.currentRow()))
+        connect_lambda(b.clicked, self, lambda self: self.edit_rule(self.table.currentRow()))
         self.bb.addButton(b, self.bb.ActionRole)
         l.addWidget(self.bb)
 
@@ -257,6 +258,12 @@ class DisplayedFields(QAbstractListModel):  # {{{
             return QIcon(I('column.png'))
         return None
 
+    def toggle_all(self, show=True):
+        for i in range(self.rowCount()):
+            idx = self.index(i)
+            if idx.isValid():
+                self.setData(idx, show, Qt.CheckStateRole)
+
     def flags(self, index):
         ans = QAbstractListModel.flags(self, index)
         return ans | Qt.ItemIsUserCheckable
@@ -428,6 +435,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('emblem_position', gprefs, choices=[
             (_('Left'), 'left'), (_('Top'), 'top'), (_('Right'), 'right'), (_('Bottom'), 'bottom')])
         r('book_list_extra_row_spacing', gprefs)
+        r('booklist_grid', gprefs)
         r('book_details_narrow_comments_layout', gprefs, choices=[(_('Float'), 'float'), (_('Columns'), 'columns')])
         self.opt_book_details_narrow_comments_layout.setToolTip(textwrap.fill(_(
             'Choose how the text is laid out when using the "Narrow" user interface layout.'
@@ -449,7 +457,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                  if l != lang]
         if lang != 'en':
             items.append(('en', get_esc_lang('en')))
-        items.sort(cmp=lambda x, y: cmp(x[1].lower(), y[1].lower()))
+        items.sort(key=lambda x: x[1].lower())
         choices = [(y, x) for x, y in items]
         # Default language is the autodetected one
         choices = [(get_language(lang), lang)] + choices
@@ -479,17 +487,17 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('tags_browser_collapse_at', gprefs)
         r('tag_browser_dont_collapse', gprefs, setting=CommaSeparatedList)
 
-        choices = set([k for k in db.field_metadata.all_field_keys()
+        choices = {k for k in db.field_metadata.all_field_keys()
                 if (db.field_metadata[k]['is_category'] and (
                     db.field_metadata[k]['datatype'] in ['text', 'series', 'enumeration'
                     ]) and not db.field_metadata[k]['display'].get('is_names', False)) or (
                     db.field_metadata[k]['datatype'] in ['composite'
-                    ] and db.field_metadata[k]['display'].get('make_category', False))])
-        choices -= set(['authors', 'publisher', 'formats', 'news', 'identifiers'])
-        choices |= set(['search'])
+                    ] and db.field_metadata[k]['display'].get('make_category', False))}
+        choices -= {'authors', 'publisher', 'formats', 'news', 'identifiers'}
+        choices |= {'search'}
         self.opt_categories_using_hierarchy.update_items_cache(choices)
         r('categories_using_hierarchy', db.prefs, setting=CommaSeparatedList,
-          choices=sorted(list(choices), key=sort_key))
+          choices=sorted(choices, key=sort_key))
 
         fm = db.field_metadata
         choices = sorted(((fm[k]['name'], k) for k in fm.displayable_field_keys() if fm[k]['name']),
@@ -503,19 +511,19 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 self.field_display_order)
         self.display_model.dataChanged.connect(self.changed_signal)
         self.field_display_order.setModel(self.display_model)
-        self.df_up_button.clicked.connect(partial(move_field_up,
-                                  self.field_display_order, self.display_model))
-        self.df_down_button.clicked.connect(partial(move_field_down,
-                                  self.field_display_order, self.display_model))
+        connect_lambda(self.df_up_button.clicked, self,
+                lambda self: move_field_up(self.field_display_order, self.display_model))
+        connect_lambda(self.df_down_button.clicked, self,
+                lambda self: move_field_down(self.field_display_order, self.display_model))
 
         self.qv_display_model = QVDisplayedFields(self.gui.current_db,
                 self.qv_display_order)
         self.qv_display_model.dataChanged.connect(self.changed_signal)
         self.qv_display_order.setModel(self.qv_display_model)
-        self.qv_up_button.clicked.connect(partial(move_field_up,
-                                  self.qv_display_order, self.qv_display_model))
-        self.qv_down_button.clicked.connect(partial(move_field_down,
-                                  self.qv_display_order, self.qv_display_model))
+        connect_lambda(self.qv_up_button.clicked, self,
+                lambda self: move_field_up(self.qv_display_order, self.qv_display_model))
+        connect_lambda(self.qv_down_button.clicked, self,
+                lambda self: move_field_down(self.qv_display_order, self.qv_display_model))
 
         self.edit_rules = EditRules(self.tabWidget)
         self.edit_rules.changed.connect(self.changed_signal)
@@ -535,8 +543,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.tabWidget.setCurrentIndex(0)
         keys = [QKeySequence('F11', QKeySequence.PortableText), QKeySequence(
             'Ctrl+Shift+F', QKeySequence.PortableText)]
-        keys = [unicode(x.toString(QKeySequence.NativeText)) for x in keys]
-        self.fs_help_msg.setText(unicode(self.fs_help_msg.text())%(
+        keys = [unicode_type(x.toString(QKeySequence.NativeText)) for x in keys]
+        self.fs_help_msg.setText(unicode_type(self.fs_help_msg.text())%(
             _(' or ').join(keys)))
         self.size_calculated.connect(self.update_cg_cache_size, type=Qt.QueuedConnection)
         self.tabWidget.currentChanged.connect(self.tab_changed)
@@ -558,8 +566,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         b.clicked.connect(self.restore_cover_grid_appearance)
         self.cover_grid_empty_cache.clicked.connect(self.empty_cache)
         self.cover_grid_open_cache.clicked.connect(self.open_cg_cache)
-        self.cover_grid_smaller_cover.clicked.connect(partial(self.resize_cover, True))
-        self.cover_grid_larger_cover.clicked.connect(partial(self.resize_cover, False))
+        connect_lambda(self.cover_grid_smaller_cover.clicked, self, lambda self: self.resize_cover(True))
+        connect_lambda(self.cover_grid_larger_cover.clicked, self, lambda self: self.resize_cover(False))
         self.cover_grid_reset_size.clicked.connect(self.cg_reset_size)
         self.opt_cover_grid_disk_cache_size.setMinimum(self.gui.grid_view.thumbnail_cache.min_disk_cache)
         self.opt_cover_grid_disk_cache_size.setMaximum(self.gui.grid_view.thumbnail_cache.min_disk_cache * 100)
@@ -721,7 +729,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def update_font_display(self):
         font = self.build_font_obj()
         fi = QFontInfo(font)
-        name = unicode(fi.family())
+        name = unicode_type(fi.family())
 
         self.font_display.setFont(font)
         self.font_display.setText(name + ' [%dpt]'%fi.pointSize())
@@ -731,7 +739,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         if fd.exec_() == fd.Accepted:
             font = fd.selectedFont()
             fi = QFontInfo(font)
-            self.current_font = [unicode(fi.family()), fi.pointSize(),
+            self.current_font = [unicode_type(fi.family()), fi.pointSize(),
                     fi.weight(), fi.italic(), font.stretch()]
             self.update_font_display()
             self.changed_signal.emit()
@@ -766,6 +774,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         gui.tags_view.set_look_and_feel()
         gui.tags_view.reread_collapse_parameters()
         gui.library_view.refresh_book_details()
+        gui.library_view.refresh_grid()
         gui.library_view.set_row_header_visibility()
         gui.cover_flow.setShowReflections(gprefs['cover_browser_reflections'])
         gui.cover_flow.setPreserveAspectRatio(gprefs['cb_preserve_aspect_ratio'])
