@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -22,6 +23,7 @@ from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.utils.icu import lower
 from calibre.constants import iswindows
+from polyglot.builtins import iteritems, itervalues, unicode_type
 
 
 class AdaptSQP(SearchQueryParser):
@@ -60,10 +62,10 @@ class PluginModel(QAbstractItemModel, AdaptSQP):  # {{{
         self.categories = sorted(self._data.keys())
 
         for plugins in self._data.values():
-            plugins.sort(cmp=lambda x, y: cmp(x.name.lower(), y.name.lower()))
+            plugins.sort(key=lambda x: x.name.lower())
 
     def universal_set(self):
-        ans = set([])
+        ans = set()
         for c, category in enumerate(self.categories):
             ans.add((c, -1))
             for p, plugin in enumerate(self._data[category]):
@@ -73,7 +75,7 @@ class PluginModel(QAbstractItemModel, AdaptSQP):  # {{{
     def get_matches(self, location, query, candidates=None):
         if candidates is None:
             candidates = self.universal_set()
-        ans = set([])
+        ans = set()
         if not query:
             return ans
         query = lower(query)
@@ -203,7 +205,7 @@ class PluginModel(QAbstractItemModel, AdaptSQP):  # {{{
             plugin = self.index_to_plugin(index)
             disabled = is_disabled(plugin)
             if role == Qt.DisplayRole:
-                ver = '.'.join(map(str, plugin.version))
+                ver = '.'.join(map(unicode_type, plugin.version))
                 desc = '\n'.join(textwrap.wrap(plugin.description, 100))
                 ans='%s (%s) %s %s\n%s'%(plugin.name, ver, _('by'), plugin.author, desc)
                 c = plugin_customization(plugin)
@@ -271,7 +273,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         if not idx.isValid():
             idx = self._plugin_model.index(0, 0)
         idx = self._plugin_model.find_next(idx,
-                unicode(self.search.currentText()))
+                unicode_type(self.search.currentText()))
         self.highlight_index(idx)
 
     def find_previous(self, *args):
@@ -279,7 +281,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         if not idx.isValid():
             idx = self._plugin_model.index(0, 0)
         idx = self._plugin_model.find_next(idx,
-            unicode(self.search.currentText()), backwards=True)
+            unicode_type(self.search.currentText()), backwards=True)
         self.highlight_index(idx)
 
     def toggle_plugin(self, *args):
@@ -317,7 +319,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 plugin = add_plugin(path)
             except NameConflict as e:
                 return error_dialog(self, _('Already exists'),
-                        unicode(e), show=True)
+                        unicode_type(e), show=True)
             self._plugin_model.beginResetModel()
             self._plugin_model.populate()
             self._plugin_model.endResetModel()
@@ -339,7 +341,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         index = self.plugin_view.currentIndex()
         if index.isValid():
             if not index.parent().isValid():
-                name = unicode(index.data() or '')
+                name = unicode_type(index.data() or '')
                 return error_dialog(self, _('Error'), '<p>'+
                         _('Select an actual plugin under <b>%s</b> to customize')%name,
                         show=True, show_copy_button=False)
@@ -377,7 +379,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                     'confirm_plugin_removal_msg', parent=self):
                     return
 
-                msg = _('Plugin <b>{0}</b> successfully removed').format(plugin.name)
+                msg = _('Plugin <b>{0}</b> successfully removed. You will have'
+                        ' to restart calibre for it to be completely removed.').format(plugin.name)
                 if remove_plugin(plugin):
                     self._plugin_model.beginResetModel()
                     self._plugin_model.populate()
@@ -432,13 +435,16 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             (key, list(gprefs.get('action-layout-'+key, [])))
             for key in all_locations])
 
+        # If this is an update, do nothing
+        if previously_installed:
+            return
         # If already installed in a GUI container, do nothing
-        for action_names in installed_actions.itervalues():
+        for action_names in itervalues(installed_actions):
             if plugin_action.name in action_names:
                 return
 
         allowed_locations = [(key, text) for key, text in
-                all_locations.iteritems() if key
+                iteritems(all_locations) if key
                 not in plugin_action.dont_add_to]
         if not allowed_locations:
             return  # This plugin doesn't want to live in the GUI
