@@ -2,8 +2,11 @@
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 # License: GPLv3 Copyright: 2010, Kovid Goyal <kovid at kovidgoyal.net>
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import errno
 import json
+import numbers
 import os
 import sys
 import textwrap
@@ -17,7 +20,7 @@ from PyQt5.Qt import (
 )
 
 from calibre import as_unicode
-from calibre.constants import isportable, iswindows
+from calibre.constants import isportable, iswindows, plugins
 from calibre.gui2 import (
     choose_files, choose_save_file, config, error_dialog, gprefs, info_dialog,
     open_url, warning_dialog
@@ -31,6 +34,7 @@ from calibre.srv.users import (
     UserManager, create_user_data, validate_password, validate_username
 )
 from calibre.utils.icu import primary_sort_key
+from polyglot.builtins import unicode_type, as_bytes
 
 try:
     from PyQt5 import sip
@@ -47,49 +51,30 @@ if iswindows and not isportable:
         return exe
 
     def startup_shortcut_path():
-        from win32com.shell import shell, shellcon
-        startup_path = shell.SHGetFolderPath(0, shellcon.CSIDL_STARTUP, 0, 0)
+        winutil = plugins['winutil'][0]
+        startup_path = winutil.special_folder_path(winutil.CSIDL_STARTUP)
         return os.path.join(startup_path, "calibre.lnk")
 
-    class Shortcut(object):
+    def create_shortcut(shortcut_path, target, description, *args):
+        quoted_args = None
+        if args:
+            quoted_args = []
+            for arg in args:
+                quoted_args.append('"{}"'.format(arg))
+            quoted_args = ' '.join(quoted_args)
+        plugins['winutil'][0].manage_shortcut(shortcut_path, target, description, quoted_args)
 
-        def __enter__(self):
-            import pythoncom
-            from win32com.shell import shell
-            pythoncom.CoInitialize()
-            self.instance = pythoncom.CoCreateInstance(shell.CLSID_ShellLink, None, pythoncom.CLSCTX_INPROC_SERVER, shell.IID_IShellLink)
-            self.persist_file = self.instance.QueryInterface(pythoncom.IID_IPersistFile)
-            return self
-
-        def __exit__(self, *a):
-            import pythoncom
-            del self.instance
-            del self.persist_file
-            pythoncom.CoUninitialize()
-
-        def create_at(self, shortcut_path, target, description, *args):
-            shortcut = self.instance
-            shortcut.SetPath(target)
-            shortcut.SetIconLocation(target, 0)
-            shortcut.SetDescription(description)
-            if args:
-                quoted_args = []
-                for arg in args:
-                    quoted_args.append('"{}"'.format(arg))
-                shortcut.SetArguments(' '.join(quoted_args))
-            self.persist_file.Save(shortcut_path, 0)
-
-        def exists_at(self, shortcut_path, target):
-            if not os.access(shortcut_path, os.R_OK):
-                return False
-            self.persist_file.Load(shortcut_path)
-            name = self.instance.GetPath(8)[0]
-            return os.path.normcase(os.path.abspath(name)) == os.path.normcase(os.path.abspath(get_exe()))
+    def shortcut_exists_at(shortcut_path, target):
+        if not os.access(shortcut_path, os.R_OK):
+            return False
+        name = plugins['winutil'][0].manage_shortcut(shortcut_path, None, None, None)
+        if name is None:
+            return False
+        return os.path.normcase(os.path.abspath(name)) == os.path.normcase(os.path.abspath(target))
 
     def set_run_at_startup(run_at_startup=True):
         if run_at_startup:
-            with Shortcut() as shortcut:
-                shortcut.create_at(startup_shortcut_path(), get_exe(), 'calibre - E-book management', '--start-in-tray')
+            create_shortcut(startup_shortcut_path(), get_exe(), 'calibre - E-book management', '--start-in-tray')
         else:
             shortcut_path = startup_shortcut_path()
             if os.path.exists(shortcut_path):
@@ -97,8 +82,7 @@ if iswindows and not isportable:
 
     def is_set_to_run_at_startup():
         try:
-            with Shortcut() as shortcut:
-                return shortcut.exists_at(startup_shortcut_path(), get_exe())
+            return shortcut_exists_at(startup_shortcut_path(), get_exe())
         except Exception:
             import traceback
             traceback.print_exc()
@@ -188,7 +172,7 @@ class Text(QLineEdit):
         return self.text().strip() or None
 
     def set(self, val):
-        self.setText(type(u'')(val or ''))
+        self.setText(unicode_type(val or ''))
 
 
 class Path(QWidget):
@@ -217,7 +201,7 @@ class Path(QWidget):
         return self.text.text().strip() or None
 
     def set(self, val):
-        self.text.setText(type(u'')(val or ''))
+        self.text.setText(unicode_type(val or ''))
 
     def choose(self):
         ans = choose_files(self, 'choose_path_srv_opts_' + self.dname, _('Choose a file'), select_only_single_file=True)
@@ -267,9 +251,9 @@ class AdvancedTab(QWidget):
                 w = Choices
             elif isinstance(opt.default, bool):
                 w = Bool
-            elif isinstance(opt.default, (int, long)):
+            elif isinstance(opt.default, numbers.Integral):
                 w = Int
-            elif isinstance(opt.default, float):
+            elif isinstance(opt.default, numbers.Real):
                 w = Float
             else:
                 w = Text
@@ -965,7 +949,7 @@ class CustomList(QWidget):  # {{{
         if path:
             raw = self.serialize(self.current_template)
             with lopen(path, 'wb') as f:
-                f.write(raw)
+                f.write(as_bytes(raw))
 
     def thumbnail_state_changed(self):
         is_enabled = bool(self.thumbnail.isChecked())
@@ -1018,7 +1002,7 @@ class CustomList(QWidget):  # {{{
         else:
             raw = self.serialize(template)
             with lopen(custom_list_template.path, 'wb') as f:
-                f.write(raw)
+                f.write(as_bytes(raw))
         return True
 
 # }}}
