@@ -7,22 +7,18 @@ import os
 import posixpath
 import sys
 import textwrap
-from binascii import hexlify
 from collections import Counter, OrderedDict, defaultdict
 from functools import partial
 
 from PyQt5.Qt import (
-    QCheckBox, QDialog, QDialogButtonBox, QFont, QFormLayout, QGridLayout, QIcon,
-    QInputDialog, QLabel, QLineEdit, QListWidget, QListWidgetItem, QMenu, QPainter,
-    QPixmap, QRadioButton, QScrollArea, QSize, QSpinBox, QStyle, QStyledItemDelegate,
-    Qt, QTimer, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
+    QApplication, QCheckBox, QDialog, QDialogButtonBox, QFont, QFormLayout,
+    QGridLayout, QIcon, QInputDialog, QLabel, QLineEdit, QListWidget,
+    QListWidgetItem, QMenu, QPainter, QPixmap, QRadioButton, QScrollArea, QSize,
+    QSpinBox, QStyle, QStyledItemDelegate, Qt, QTimer, QTreeWidget, QTreeWidgetItem,
+    QVBoxLayout, QWidget, pyqtSignal
 )
-try:
-    from PyQt5 import sip
-except ImportError:
-    import sip
 
-from calibre import human_readable, plugins, sanitize_file_name_unicode
+from calibre import human_readable, plugins, sanitize_file_name
 from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES
 from calibre.ebooks.oeb.polish.container import OEB_FONTS, guess_type
 from calibre.ebooks.oeb.polish.cover import (
@@ -42,7 +38,16 @@ from calibre.gui2.tweak_book import (
 from calibre.gui2.tweak_book.editor import syntax_from_mime
 from calibre.gui2.tweak_book.templates import template_for
 from calibre.utils.icu import numeric_sort_key
+from polyglot.builtins import iteritems, itervalues, unicode_type, range, filter, map
+from polyglot.binary import as_hex_unicode
 
+try:
+    from PyQt5 import sip
+except ImportError:
+    import sip
+
+
+FILE_COPY_MIME = 'application/calibre-edit-book-files'
 TOP_ICON_SIZE = 24
 NAME_ROLE = Qt.UserRole
 CATEGORY_ROLE = NAME_ROLE + 1
@@ -68,7 +73,7 @@ def name_is_ok(name, show_error):
     norm = name.replace('\\', '/')
     parts = name.split('/')
     for x in parts:
-        if sanitize_file_name_unicode(x) != x:
+        if sanitize_file_name(x) != x:
             return show_error(_('The file name contains invalid characters')) and False
     if current_container().has_name(norm):
         return show_error(_('This file name already exists in the book')) and False
@@ -76,7 +81,7 @@ def name_is_ok(name, show_error):
     return True
 
 
-def get_bulk_rename_settings(parent, number, msg=None, sanitize=sanitize_file_name_unicode,
+def get_bulk_rename_settings(parent, number, msg=None, sanitize=sanitize_file_name,
         leading_zeros=True, prefix=None, category='text', allow_spine_order=False):  # {{{
     d = QDialog(parent)
     d.setWindowTitle(_('Bulk rename items'))
@@ -106,14 +111,14 @@ def get_bulk_rename_settings(parent, number, msg=None, sanitize=sanitize_file_na
     ans = {'prefix': None, 'start': None}
 
     if d.exec_() == d.Accepted:
-        prefix = sanitize(unicode(d.prefix.text()))
+        prefix = sanitize(unicode_type(d.prefix.text()))
         previous[category] = prefix
         tprefs.set('file-list-bulk-rename-prefix', previous)
         num = d.num.value()
         fmt = '%d'
         if leading_zeros:
             largest = num + number - 1
-            fmt = '%0{0}d'.format(len(str(largest)))
+            fmt = '%0{0}d'.format(len(unicode_type(largest)))
         ans['prefix'] = prefix + fmt
         ans['start'] = num
         if allow_spine_order:
@@ -127,7 +132,7 @@ class ItemDelegate(QStyledItemDelegate):  # {{{
     rename_requested = pyqtSignal(object, object)
 
     def setEditorData(self, editor, index):
-        name = unicode(index.data(NAME_ROLE) or '')
+        name = unicode_type(index.data(NAME_ROLE) or '')
         # We do this because Qt calls selectAll() unconditionally on the
         # editor, and we want only a part of the file name to be selected
         QTimer.singleShot(0, partial(self.set_editor_data, name, editor))
@@ -146,8 +151,8 @@ class ItemDelegate(QStyledItemDelegate):  # {{{
             editor.selectAll()
 
     def setModelData(self, editor, model, index):
-        newname = unicode(editor.text())
-        oldname = unicode(index.data(NAME_ROLE) or '')
+        newname = unicode_type(editor.text())
+        oldname = unicode_type(index.data(NAME_ROLE) or '')
         if newname != oldname:
             self.rename_requested.emit(oldname, newname)
 
@@ -165,7 +170,7 @@ class ItemDelegate(QStyledItemDelegate):  # {{{
                 suffix = '%s(%d)' % (NBSP, index.model().rowCount(index))
             else:
                 try:
-                    suffix = NBSP + human_readable(current_container().filesize(unicode(index.data(NAME_ROLE) or '')))
+                    suffix = NBSP + human_readable(current_container().filesize(unicode_type(index.data(NAME_ROLE) or '')))
                 except EnvironmentError:
                     suffix = NBSP + human_readable(0)
             br = painter.boundingRect(option.rect, Qt.AlignRight|Qt.AlignVCenter, suffix)
@@ -194,6 +199,8 @@ class FileList(QTreeWidget):
     export_requested = pyqtSignal(object, object)
     replace_requested = pyqtSignal(object, object, object, object)
     link_stylesheets_requested = pyqtSignal(object, object, object)
+    initiate_file_copy = pyqtSignal(object)
+    initiate_file_paste = pyqtSignal()
 
     def __init__(self, parent=None):
         QTreeWidget.__init__(self, parent)
@@ -228,13 +235,13 @@ class FileList(QTreeWidget):
         self.rendered_emblem_cache = {}
         self.top_level_pixmap_cache = {
             name : QIcon(I(icon)).pixmap(TOP_ICON_SIZE, TOP_ICON_SIZE)
-            for name, icon in {
+            for name, icon in iteritems({
                 'text':'keyboard-prefs.png',
                 'styles':'lookfeel.png',
                 'fonts':'font.png',
                 'misc':'mimetypes/dir.png',
                 'images':'view-image.png',
-            }.iteritems()}
+            })}
         self.itemActivated.connect(self.item_double_clicked)
 
     def mimeTypes(self):
@@ -252,36 +259,36 @@ class FileList(QTreeWidget):
     def current_name(self):
         ci = self.currentItem()
         if ci is not None:
-            return unicode(ci.data(0, NAME_ROLE) or '')
+            return unicode_type(ci.data(0, NAME_ROLE) or '')
         return ''
 
     def get_state(self):
         s = {'pos':self.verticalScrollBar().value()}
-        s['expanded'] = {c for c, item in self.categories.iteritems() if item.isExpanded()}
-        s['selected'] = {unicode(i.data(0, NAME_ROLE) or '') for i in self.selectedItems()}
+        s['expanded'] = {c for c, item in iteritems(self.categories) if item.isExpanded()}
+        s['selected'] = {unicode_type(i.data(0, NAME_ROLE) or '') for i in self.selectedItems()}
         return s
 
     def set_state(self, state):
-        for category, item in self.categories.iteritems():
+        for category, item in iteritems(self.categories):
             item.setExpanded(category in state['expanded'])
         self.verticalScrollBar().setValue(state['pos'])
-        for parent in self.categories.itervalues():
-            for c in (parent.child(i) for i in xrange(parent.childCount())):
-                name = unicode(c.data(0, NAME_ROLE) or '')
+        for parent in itervalues(self.categories):
+            for c in (parent.child(i) for i in range(parent.childCount())):
+                name = unicode_type(c.data(0, NAME_ROLE) or '')
                 if name in state['selected']:
                     c.setSelected(True)
 
     def item_from_name(self, name):
-        for parent in self.categories.itervalues():
-            for c in (parent.child(i) for i in xrange(parent.childCount())):
-                q = unicode(c.data(0, NAME_ROLE) or '')
+        for parent in itervalues(self.categories):
+            for c in (parent.child(i) for i in range(parent.childCount())):
+                q = unicode_type(c.data(0, NAME_ROLE) or '')
                 if q == name:
                     return c
 
     def select_name(self, name, set_as_current_index=False):
-        for parent in self.categories.itervalues():
-            for c in (parent.child(i) for i in xrange(parent.childCount())):
-                q = unicode(c.data(0, NAME_ROLE) or '')
+        for parent in itervalues(self.categories):
+            for c in (parent.child(i) for i in range(parent.childCount())):
+                q = unicode_type(c.data(0, NAME_ROLE) or '')
                 c.setSelected(q == name)
                 if q == name:
                     self.scrollToItem(c)
@@ -289,9 +296,9 @@ class FileList(QTreeWidget):
                         self.setCurrentItem(c)
 
     def select_names(self, names, current_name=None):
-        for parent in self.categories.itervalues():
-            for c in (parent.child(i) for i in xrange(parent.childCount())):
-                q = unicode(c.data(0, NAME_ROLE) or '')
+        for parent in itervalues(self.categories):
+            for c in (parent.child(i) for i in range(parent.childCount())):
+                q = unicode_type(c.data(0, NAME_ROLE) or '')
                 c.setSelected(q in names)
                 if q == current_name:
                     self.scrollToItem(c)
@@ -348,7 +355,7 @@ class FileList(QTreeWidget):
         cover_page_name = get_cover_page_name(container)
         cover_image_name = get_raster_cover_name(container)
         manifested_names = set()
-        for names in container.manifest_type_map.itervalues():
+        for names in itervalues(container.manifest_type_map):
             manifested_names |= set(names)
 
         def get_category(name, mt):
@@ -385,7 +392,7 @@ class FileList(QTreeWidget):
 
             seen[text] = item
             item.setText(0, text)
-            item.setText(1, hexlify(numeric_sort_key(text)))
+            item.setText(1, as_hex_unicode(numeric_sort_key(text)))
 
         def render_emblems(item, emblems):
             emblems = tuple(emblems)
@@ -474,7 +481,7 @@ class FileList(QTreeWidget):
                 continue
             processed[name] = create_item(name)
 
-        for name, c in self.categories.iteritems():
+        for name, c in iteritems(self.categories):
             c.setExpanded(True)
             if name != 'text':
                 c.sortChildren(1, Qt.AscendingOrder)
@@ -489,7 +496,7 @@ class FileList(QTreeWidget):
 
     def show_context_menu(self, point):
         item = self.itemAt(point)
-        if item is None or item in set(self.categories.itervalues()):
+        if item is None or item in tuple(itervalues(self.categories)):
             return
         m = QMenu(self)
         sel = self.selectedItems()
@@ -497,9 +504,9 @@ class FileList(QTreeWidget):
         container = current_container()
         ci = self.currentItem()
         if ci is not None:
-            cn = unicode(ci.data(0, NAME_ROLE) or '')
-            mt = unicode(ci.data(0, MIME_ROLE) or '')
-            cat = unicode(ci.data(0, CATEGORY_ROLE) or '')
+            cn = unicode_type(ci.data(0, NAME_ROLE) or '')
+            mt = unicode_type(ci.data(0, MIME_ROLE) or '')
+            cat = unicode_type(ci.data(0, CATEGORY_ROLE) or '')
             n = elided_text(cn.rpartition('/')[-1])
             m.addAction(QIcon(I('save.png')), _('Export %s') % n, partial(self.export, cn))
             if cn not in container.names_that_must_not_be_changed and cn not in container.names_that_must_not_be_removed and mt not in OEB_FONTS:
@@ -523,13 +530,19 @@ class FileList(QTreeWidget):
             m.addAction(QIcon(I('modified.png')), _('Change the file extension for the selected files'), self.request_change_ext)
             m.addAction(QIcon(I('trash.png')), ngettext(
                 '&Delete the selected file', '&Delete the {} selected files', num).format(num), self.request_delete)
+            m.addAction(QIcon(I('edit-copy.png')), ngettext(
+                '&Copy the selected file to another editor instance',
+                '&Copy the {} selected files to another editor instance', num).format(num), self.copy_selected_files)
             m.addSeparator()
+        md = QApplication.instance().clipboard().mimeData()
+        if md.hasUrls() and md.hasFormat(FILE_COPY_MIME):
+            m.addAction(_('Paste files from other editor instance'), self.paste_from_other_instance)
 
         selected_map = defaultdict(list)
         for item in sel:
-            selected_map[unicode(item.data(0, CATEGORY_ROLE) or '')].append(unicode(item.data(0, NAME_ROLE) or ''))
+            selected_map[unicode_type(item.data(0, CATEGORY_ROLE) or '')].append(unicode_type(item.data(0, NAME_ROLE) or ''))
 
-        for items in selected_map.itervalues():
+        for items in itervalues(selected_map):
             items.sort(key=self.index_of_name)
 
         if selected_map['text']:
@@ -544,10 +557,10 @@ class FileList(QTreeWidget):
             m.popup(self.mapToGlobal(point))
 
     def index_of_name(self, name):
-        for category, parent in self.categories.iteritems():
-            for i in xrange(parent.childCount()):
+        for category, parent in iteritems(self.categories):
+            for i in range(parent.childCount()):
                 item = parent.child(i)
-                if unicode(item.data(0, NAME_ROLE) or '') == name:
+                if unicode_type(item.data(0, NAME_ROLE) or '') == name:
                     return (category, i)
         return (None, -1)
 
@@ -570,7 +583,7 @@ class FileList(QTreeWidget):
         self.mark_requested.emit(name, 'cover')
 
     def mark_as_titlepage(self, name):
-        first = unicode(self.categories['text'].child(0).data(0, NAME_ROLE) or '') == name
+        first = unicode_type(self.categories['text'].child(0).data(0, NAME_ROLE) or '') == name
         move_to_start = False
         if not first:
             move_to_start = question_dialog(self, _('Not first item'), _(
@@ -593,7 +606,7 @@ class FileList(QTreeWidget):
                 ' internally. The filenames you see are automatically generated from the'
                 ' internal structures of the original file.') % current_container().book_type.upper(), show=True)
             return
-        names = {unicode(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()}
+        names = {unicode_type(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()}
         bad = names & current_container().names_that_must_not_be_changed
         if bad:
             error_dialog(self, _('Cannot rename'),
@@ -605,7 +618,7 @@ class FileList(QTreeWidget):
     def request_bulk_rename(self):
         names = self.request_rename_common()
         if names is not None:
-            categories = Counter(unicode(item.data(0, CATEGORY_ROLE) or '') for item in self.selectedItems())
+            categories = Counter(unicode_type(item.data(0, CATEGORY_ROLE) or '') for item in self.selectedItems())
             settings = get_bulk_rename_settings(self, len(names), category=categories.most_common(1)[0][0], allow_spine_order=True)
             fmt, num = settings['prefix'], settings['start']
             if fmt is not None:
@@ -640,10 +653,16 @@ class FileList(QTreeWidget):
 
     @property
     def selected_names(self):
-        ans = {unicode(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()}
+        ans = {unicode_type(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()}
         ans.discard('')
         return ans
 
+    def copy_selected_files(self):
+        self.initiate_file_copy.emit(self.selected_names)
+
+    def paste_from_other_instance(self):
+        self.initiate_file_paste.emit()
+
     def request_delete(self):
         names = self.selected_names
         bad = names & current_container().names_that_must_not_be_removed
@@ -652,10 +671,10 @@ class FileList(QTreeWidget):
                          _('The file(s) %s cannot be deleted.') % ('<b>%s</b>' % ', '.join(bad)), show=True)
 
         text = self.categories['text']
-        children = (text.child(i) for i in xrange(text.childCount()))
-        spine_removals = [(unicode(item.data(0, NAME_ROLE) or ''), item.isSelected()) for item in children]
-        other_removals = {unicode(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()
-                          if unicode(item.data(0, CATEGORY_ROLE) or '') != 'text'}
+        children = (text.child(i) for i in range(text.childCount()))
+        spine_removals = [(unicode_type(item.data(0, NAME_ROLE) or ''), item.isSelected()) for item in children]
+        other_removals = {unicode_type(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()
+                          if unicode_type(item.data(0, CATEGORY_ROLE) or '') != 'text'}
         self.delete_requested.emit(spine_removals, other_removals)
 
     def delete_done(self, spine_removals, other_removals):
@@ -663,11 +682,11 @@ class FileList(QTreeWidget):
         for i, (name, remove) in enumerate(spine_removals):
             if remove:
                 removals.append(self.categories['text'].child(i))
-        for category, parent in self.categories.iteritems():
+        for category, parent in iteritems(self.categories):
             if category != 'text':
-                for i in xrange(parent.childCount()):
+                for i in range(parent.childCount()):
                     child = parent.child(i)
-                    if unicode(child.data(0, NAME_ROLE) or '') in other_removals:
+                    if unicode_type(child.data(0, NAME_ROLE) or '') in other_removals:
                         removals.append(child)
 
         # The sorting by index is necessary otherwise Qt crashes with recursive
@@ -698,13 +717,13 @@ class FileList(QTreeWidget):
     def dropEvent(self, event):
         with self:
             text = self.categories['text']
-            pre_drop_order = {text.child(i):i for i in xrange(text.childCount())}
+            pre_drop_order = {text.child(i):i for i in range(text.childCount())}
             super(FileList, self).dropEvent(event)
-            current_order = {text.child(i):i for i in xrange(text.childCount())}
+            current_order = {text.child(i):i for i in range(text.childCount())}
             if current_order != pre_drop_order:
                 order = []
-                for child in (text.child(i) for i in xrange(text.childCount())):
-                    name = unicode(child.data(0, NAME_ROLE) or '')
+                for child in (text.child(i) for i in range(text.childCount())):
+                    name = unicode_type(child.data(0, NAME_ROLE) or '')
                     linear = bool(child.data(0, LINEAR_ROLE))
                     order.append([name, linear])
                 # Ensure that all non-linear items are at the end, any non-linear
@@ -715,14 +734,14 @@ class FileList(QTreeWidget):
                 self.reorder_spine.emit(order)
 
     def item_double_clicked(self, item, column):
-        category = unicode(item.data(0, CATEGORY_ROLE) or '')
+        category = unicode_type(item.data(0, CATEGORY_ROLE) or '')
         if category:
             self._request_edit(item)
 
     def _request_edit(self, item):
-        category = unicode(item.data(0, CATEGORY_ROLE) or '')
-        mime = unicode(item.data(0, MIME_ROLE) or '')
-        name = unicode(item.data(0, NAME_ROLE) or '')
+        category = unicode_type(item.data(0, CATEGORY_ROLE) or '')
+        mime = unicode_type(item.data(0, MIME_ROLE) or '')
+        name = unicode_type(item.data(0, NAME_ROLE) or '')
         syntax = {'text':'html', 'styles':'css'}.get(category, None)
         self.edit_file.emit(name, syntax, mime)
 
@@ -737,11 +756,11 @@ class FileList(QTreeWidget):
     def edit_next_file(self, currently_editing=None, backwards=False):
         category = self.categories['text']
         seen_current = False
-        items = (category.child(i) for i in xrange(category.childCount()))
+        items = (category.child(i) for i in range(category.childCount()))
         if backwards:
             items = reversed(tuple(items))
         for item in items:
-            name = unicode(item.data(0, NAME_ROLE) or '')
+            name = unicode_type(item.data(0, NAME_ROLE) or '')
             if seen_current:
                 self._request_edit(item)
                 return True
@@ -751,15 +770,15 @@ class FileList(QTreeWidget):
 
     @property
     def all_files(self):
-        return (category.child(i) for category in self.categories.itervalues() for i in xrange(category.childCount()))
+        return (category.child(i) for category in itervalues(self.categories) for i in range(category.childCount()))
 
     @property
     def searchable_names(self):
         ans = {'text':OrderedDict(), 'styles':OrderedDict(), 'selected':OrderedDict(), 'open':OrderedDict()}
         for item in self.all_files:
-            category = unicode(item.data(0, CATEGORY_ROLE) or '')
-            mime = unicode(item.data(0, MIME_ROLE) or '')
-            name = unicode(item.data(0, NAME_ROLE) or '')
+            category = unicode_type(item.data(0, CATEGORY_ROLE) or '')
+            mime = unicode_type(item.data(0, MIME_ROLE) or '')
+            name = unicode_type(item.data(0, NAME_ROLE) or '')
             ok = category in {'text', 'styles'}
             if ok:
                 ans[category][name] = syntax_from_mime(name, mime)
@@ -819,7 +838,7 @@ class FileList(QTreeWidget):
 
     def link_stylesheets(self, names):
         s = self.categories['styles']
-        sheets = [unicode(s.child(i).data(0, NAME_ROLE) or '') for i in xrange(s.childCount())]
+        sheets = [unicode_type(s.child(i).data(0, NAME_ROLE) or '') for i in range(s.childCount())]
         if not sheets:
             return error_dialog(self, _('No stylesheets'), _(
                 'This book currently has no stylesheets. You must first create a stylesheet'
@@ -852,7 +871,7 @@ class FileList(QTreeWidget):
         l.addWidget(bb)
         if d.exec_() == d.Accepted:
             tprefs['remove_existing_links_when_linking_sheets'] = r.isChecked()
-            sheets = [unicode(s.item(il).text()) for il in xrange(s.count()) if s.item(il).checkState() == Qt.Checked]
+            sheets = [unicode_type(s.item(il).text()) for il in range(s.count()) if s.item(il).checkState() == Qt.Checked]
             if sheets:
                 self.link_stylesheets_requested.emit(names, sheets, r.isChecked())
 
@@ -918,7 +937,7 @@ class NewFileDialog(QDialog):  # {{{
 
     @property
     def name_is_ok(self):
-        return name_is_ok(unicode(self.name.text()), self.show_error)
+        return name_is_ok(unicode_type(self.name.text()), self.show_error)
 
     def update_ok(self, *args):
         self.ok_button.setEnabled(self.name_is_ok)
@@ -928,7 +947,7 @@ class NewFileDialog(QDialog):  # {{{
             return error_dialog(self, _('No name specified'), _(
                 'You must specify a name for the new file, with an extension, for example, chapter1.html'), show=True)
         tprefs['auto_link_stylesheets'] = self.link_css.isChecked()
-        name = unicode(self.name.text())
+        name = unicode_type(self.name.text())
         name, ext = name.rpartition('.')[0::2]
         name = (name + '.' + ext.lower()).replace('\\', '/')
         mt = guess_type(name)
@@ -971,7 +990,7 @@ class MergeDialog(QDialog):  # {{{
 
         buttons = self.buttons = [QRadioButton(n) for n in names]
         buttons[0].setChecked(True)
-        map(w.l.addWidget, buttons)
+        tuple(map(w.l.addWidget, buttons))
         sa.setWidget(w)
 
         self.resize(self.sizeHint() + QSize(150, 20))
@@ -993,7 +1012,7 @@ class FileListWidget(QWidget):
         self.file_list = FileList(self)
         self.layout().addWidget(self.file_list)
         self.layout().setContentsMargins(0, 0, 0, 0)
-        self.forwarded_signals = {k for k, o in vars(self.file_list.__class__).iteritems() if isinstance(o, pyqtSignal) and '_' in k and not hasattr(self, k)}
+        self.forwarded_signals = {k for k, o in iteritems(vars(self.file_list.__class__)) if isinstance(o, pyqtSignal) and '_' in k and not hasattr(self, k)}
         for x in ('delete_done', 'select_name', 'select_names', 'request_edit', 'mark_name_as_current', 'clear_currently_edited_name'):
             setattr(self, x, getattr(self.file_list, x))
         self.setFocusProxy(self.file_list)
@@ -1011,6 +1030,6 @@ class FileListWidget(QWidget):
         return self.file_list.current_name
 
     def __getattr__(self, name):
-        if name in self.forwarded_signals:
+        if name in object.__getattribute__(self, 'forwarded_signals'):
             return getattr(self.file_list, name)
         return QWidget.__getattr__(self, name)
