@@ -1,14 +1,12 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import time, textwrap, json
 from bisect import bisect_right
-from polyglot.builtins import map, unicode_type, filter
 from threading import Thread
 from functools import partial
 
@@ -28,6 +26,7 @@ from calibre.gui2.viewer.documentview import apply_settings
 from calibre.gui2.viewer.config import config
 from calibre.gui2.widgets2 import HistoryLineEdit2
 from calibre.utils.ipc.simple_worker import offload_worker
+from polyglot.builtins import filter, map, native_string_type, unicode_type
 from polyglot.urllib import urlparse
 from polyglot.queue import Queue, Empty
 from polyglot.binary import as_base64_unicode
@@ -167,7 +166,7 @@ class NetworkReply(QNetworkReply):
             QTimer.singleShot(0, self.check_for_parse)
         else:
             data = get_data(name)
-            if isinstance(data, type('')):
+            if isinstance(data, unicode_type):
                 data = data.encode('utf-8')
                 mime_type += '; charset=utf-8'
             self.__data = data
@@ -296,7 +295,7 @@ class WebPage(QWebPage):
         mf.addToJavaScriptWindowObject("py_bridge", self)
         mf.evaluateJavaScript(self.js)
 
-    @pyqtSlot(str, str, str)
+    @pyqtSlot(native_string_type, native_string_type, native_string_type)
     def request_sync(self, tag_name, href, sourceline_address):
         try:
             self.sync_requested.emit(unicode_type(tag_name), unicode_type(href), json.loads(unicode_type(sourceline_address)))
@@ -305,9 +304,9 @@ class WebPage(QWebPage):
 
     def go_to_anchor(self, anchor, lnum):
         self.mainFrame().evaluateJavaScript('window.calibre_preview_integration.go_to_anchor(%s, %s)' % (
-            json.dumps(anchor), json.dumps(str(lnum))))
+            json.dumps(anchor), json.dumps(unicode_type(lnum))))
 
-    @pyqtSlot(str, str)
+    @pyqtSlot(native_string_type, native_string_type)
     def request_split(self, loc, totals):
         actions['split-in-preview'].setChecked(False)
         loc, totals = json.loads(unicode_type(loc)), json.loads(unicode_type(totals))
@@ -370,17 +369,16 @@ class WebView(QWebView):
     def refresh(self):
         self.pageAction(self.page().Reload).trigger()
 
-    @dynamic_property
+    @property
     def scroll_pos(self):
-        def fget(self):
-            mf = self.page().mainFrame()
-            return (mf.scrollBarValue(Qt.Horizontal), mf.scrollBarValue(Qt.Vertical))
+        mf = self.page().mainFrame()
+        return (mf.scrollBarValue(Qt.Horizontal), mf.scrollBarValue(Qt.Vertical))
 
-        def fset(self, val):
-            mf = self.page().mainFrame()
-            mf.setScrollBarValue(Qt.Horizontal, val[0])
-            mf.setScrollBarValue(Qt.Vertical, val[1])
-        return property(fget=fget, fset=fset)
+    @scroll_pos.setter
+    def scroll_pos(self, val):
+        mf = self.page().mainFrame()
+        mf.setScrollBarValue(Qt.Horizontal, val[0])
+        mf.setScrollBarValue(Qt.Vertical, val[1])
 
     def clear(self):
         self.setHtml(_(
