@@ -1,4 +1,5 @@
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''
@@ -23,6 +24,7 @@ from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndica
 from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
     image_extensions, dnd_has_extension, DownloadDialog)
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import native_string_type, unicode_type, range
 
 history = XMLConfig('history')
 
@@ -44,11 +46,11 @@ class ProgressIndicator(QWidget):  # {{{
         pwidth, pheight = view.size().width(), view.size().height()
         self.resize(pwidth, min(pheight, 250))
         if self.pos is None:
-            self.move(0, (pheight-self.size().height())/2.)
+            self.move(0, int((pheight-self.size().height())/2))
         else:
             self.move(self.pos[0], self.pos[1])
         self.pi.resize(self.pi.sizeHint())
-        self.pi.move(int((self.size().width()-self.pi.size().width())/2.), 0)
+        self.pi.move(int((self.size().width()-self.pi.size().width())/2), 0)
         self.status.resize(self.size().width(), self.size().height()-self.pi.size().height()-10)
         self.status.move(0, self.pi.size().height()+10)
         self.status.setText('<h1>'+msg+'</h1>')
@@ -83,7 +85,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         # Get all items in the combobox. If we are reseting
         # to defaults we don't want to lose what the user
         # has added.
-        val_hist = [unicode(self.re.lineEdit().text())] + [unicode(self.re.itemText(i)) for i in xrange(self.re.count())]
+        val_hist = [unicode_type(self.re.lineEdit().text())] + [unicode_type(self.re.itemText(i)) for i in range(self.re.count())]
         self.re.clear()
 
         if defaults:
@@ -106,7 +108,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
     def do_test(self):
         from calibre.ebooks.metadata import authors_to_string
         from calibre.ebooks.metadata.meta import metadata_from_filename
-        fname = unicode(self.filename.text())
+        fname = unicode_type(self.filename.text())
         ext = os.path.splitext(fname)[1][1:].lower()
         if ext not in BOOK_EXTENSIONS:
             return warning_dialog(self, _('Test file name invalid'),
@@ -136,7 +138,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
             self.series.setText(_('No match'))
 
         if mi.series_index is not None:
-            self.series_index.setText(str(mi.series_index))
+            self.series_index.setText(unicode_type(mi.series_index))
         else:
             self.series_index.setText(_('No match'))
 
@@ -150,11 +152,11 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         else:
             self.pubdate.setText(_('No match'))
 
-        self.isbn.setText(_('No match') if mi.isbn is None else str(mi.isbn))
+        self.isbn.setText(_('No match') if mi.isbn is None else unicode_type(mi.isbn))
         self.comments.setText(mi.comments if mi.comments else _('No match'))
 
     def pattern(self):
-        pat = unicode(self.re.lineEdit().text())
+        pat = unicode_type(self.re.lineEdit().text())
         return re.compile(pat)
 
     def commit(self):
@@ -162,7 +164,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         prefs['filename_pattern'] = pat
 
         history = []
-        history_pats = [unicode(self.re.lineEdit().text())] + [unicode(self.re.itemText(i)) for i in xrange(self.re.count())]
+        history_pats = [unicode_type(self.re.lineEdit().text())] + [unicode_type(self.re.itemText(i)) for i in range(self.re.count())]
         for p in history_pats[:24]:
             # Ensure we don't have duplicate items.
             if p and p not in history:
@@ -303,7 +305,7 @@ def draw_size(p, rect, w, h):
     f = p.font()
     f.setBold(True)
     p.setFont(f)
-    sz = u'\u00a0%d x %d\u00a0'%(w, h)
+    sz = '\u00a0%d x %d\u00a0'%(w, h)
     flags = Qt.AlignBottom|Qt.AlignRight|Qt.TextSingleLine
     szrect = p.boundingRect(rect, flags, sz)
     p.fillRect(szrect.adjusted(0, 0, 0, 4), QColor(0, 0, 0, 200))
@@ -368,8 +370,8 @@ class ImageView(QWidget, ImageDropMixin):
             pmap = pmap.scaled(int(nw*pmap.devicePixelRatio()), int(nh*pmap.devicePixelRatio()), Qt.IgnoreAspectRatio,
                     Qt.SmoothTransformation)
         w, h = int(pmap.width()/pmap.devicePixelRatio()), int(pmap.height()/pmap.devicePixelRatio())
-        x = int(abs(cw - w)/2.)
-        y = int(abs(ch - h)/2.)
+        x = int(abs(cw - w)/2)
+        y = int(abs(ch - h)/2)
         target = QRect(x, y, w, h)
         p = QPainter(self)
         p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
@@ -485,23 +487,23 @@ class LineEditECM(object):  # {{{
 
     def upper_case(self):
         from calibre.utils.icu import upper
-        self.setText(upper(unicode(self.text())))
+        self.setText(upper(unicode_type(self.text())))
 
     def lower_case(self):
         from calibre.utils.icu import lower
-        self.setText(lower(unicode(self.text())))
+        self.setText(lower(unicode_type(self.text())))
 
     def swap_case(self):
         from calibre.utils.icu import swapcase
-        self.setText(swapcase(unicode(self.text())))
+        self.setText(swapcase(unicode_type(self.text())))
 
     def title_case(self):
         from calibre.utils.titlecase import titlecase
-        self.setText(titlecase(unicode(self.text())))
+        self.setText(titlecase(unicode_type(self.text())))
 
     def capitalize(self):
         from calibre.utils.icu import capitalize
-        self.setText(capitalize(unicode(self.text())))
+        self.setText(capitalize(unicode_type(self.text())))
 
 # }}}
 
@@ -541,7 +543,7 @@ class ItemsCompleter(QCompleter):  # {{{
         self.setModel(model)
 
         self.setCompletionPrefix(completion_prefix)
-        if completion_prefix.strip() != '':
+        if completion_prefix.strip():
             self.complete()
 
     def update_items_cache(self, items):
@@ -569,7 +571,7 @@ class CompleteLineEdit(EnLineEdit):  # {{{
         self.completer = ItemsCompleter(self, complete_items)
         self.completer.setCaseSensitivity(Qt.CaseInsensitive)
 
-        self.completer.activated[str].connect(self.complete_text)
+        self.completer.activated[native_string_type].connect(self.complete_text)
 
         self.completer.setWidget(self)
 
@@ -583,22 +585,22 @@ class CompleteLineEdit(EnLineEdit):  # {{{
         self.space_before_sep = space_before
 
     def text_changed(self, text):
-        all_text = unicode(text)
+        all_text = unicode_type(text)
         text = all_text[:self.cursorPosition()]
         prefix = text.split(self.separator)[-1].strip()
 
         text_items = []
         for t in all_text.split(self.separator):
-            t1 = unicode(t).strip()
-            if t1 != '':
+            t1 = unicode_type(t).strip()
+            if t1:
                 text_items.append(t)
         text_items = list(set(text_items))
         self.completer.update(text_items, prefix)
 
     def complete_text(self, text):
         cursor_pos = self.cursorPosition()
-        before_text = unicode(self.text())[:cursor_pos]
-        after_text = unicode(self.text())[cursor_pos:]
+        before_text = unicode_type(self.text())[:cursor_pos]
+        after_text = unicode_type(self.text())[cursor_pos:]
         prefix_len = len(before_text.split(self.separator)[-1].lstrip())
         if self.space_before_sep:
             complete_text_pat = '%s%s %s %s'
@@ -627,7 +629,7 @@ class EnComboBox(QComboBox):  # {{{
         self.setMinimumContentsLength(20)
 
     def text(self):
-        return unicode(self.currentText())
+        return unicode_type(self.currentText())
 
     def setText(self, text):
         idx = self.findText(text, Qt.MatchFixedString|Qt.MatchCaseSensitive)
@@ -683,11 +685,11 @@ class HistoryLineEdit(QComboBox):  # {{{
 
     def save_history(self):
         items = []
-        ct = unicode(self.currentText())
+        ct = unicode_type(self.currentText())
         if ct:
             items.append(ct)
         for i in range(self.count()):
-            item = unicode(self.itemText(i))
+            item = unicode_type(self.itemText(i))
             if item not in items:
                 items.append(item)
         self.blockSignals(True)
@@ -887,13 +889,13 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
         self.setFormat(0, textLength,
                        PythonHighlighter.Formats["normal"])
 
-        if text.startswith(u"Traceback") or text.startswith(u"Error: "):
+        if text.startswith("Traceback") or text.startswith("Error: "):
             self.setCurrentBlockState(ERROR)
             self.setFormat(0, textLength,
                            PythonHighlighter.Formats["error"])
             return
         if prevState == ERROR and \
-           not (text.startswith(u'>>>') or text.startswith(u"#")):
+           not (text.startswith('>>>') or text.startswith("#")):
             self.setCurrentBlockState(ERROR)
             self.setFormat(0, textLength,
                            PythonHighlighter.Formats["error"])
@@ -912,18 +914,18 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
         # PythonHighlighter.Rules.append((QRegExp(r"#.*"), "comment"))
         if not text:
             pass
-        elif text[0] == u"#":
+        elif text[0] == "#":
             self.setFormat(0, len(text),
                            PythonHighlighter.Formats["comment"])
         else:
             stack = []
             for i, c in enumerate(text):
-                if c in (u'"', u"'"):
+                if c in ('"', "'"):
                     if stack and stack[-1] == c:
                         stack.pop()
                     else:
                         stack.append(c)
-                elif c == u"#" and len(stack) == 0:
+                elif c == "#" and len(stack) == 0:
                     self.setFormat(i, len(text),
                                    PythonHighlighter.Formats["comment"])
                     break
@@ -1081,7 +1083,7 @@ class Splitter(QSplitter):
                 parent.addAction(self.action_toggle)
                 if hasattr(parent, 'keyboard'):
                     parent.keyboard.register_shortcut('splitter %s %s'%(name,
-                        label), unicode(self.action_toggle.text()),
+                        label), unicode_type(self.action_toggle.text()),
                         default_keys=(shortcut,), action=self.action_toggle)
                 else:
                     self.action_toggle.setShortcut(shortcut)
@@ -1130,30 +1132,28 @@ class Splitter(QSplitter):
             print(self.save_name, 'side:', self.side_index_size, 'other:', end=' ')
             print(list(self.sizes())[self.other_index])
 
-    @dynamic_property
+    @property
     def side_index_size(self):
-        def fget(self):
-            if self.count() < 2:
-                return 0
-            return self.sizes()[self.side_index]
+        if self.count() < 2:
+            return 0
+        return self.sizes()[self.side_index]
 
-        def fset(self, val):
-            if self.count() < 2:
-                return
-            if val == 0 and not self.is_side_index_hidden:
-                self.save_state()
-            sizes = list(self.sizes())
-            for i in range(len(sizes)):
-                sizes[i] = val if i == self.side_index else 10
-            self.setSizes(sizes)
-            total = sum(self.sizes())
-            sizes = list(self.sizes())
-            for i in range(len(sizes)):
-                sizes[i] = val if i == self.side_index else total-val
-            self.setSizes(sizes)
-            self.initialize()
-
-        return property(fget=fget, fset=fset)
+    @side_index_size.setter
+    def side_index_size(self, val):
+        if self.count() < 2:
+            return
+        if val == 0 and not self.is_side_index_hidden:
+            self.save_state()
+        sizes = list(self.sizes())
+        for i in range(len(sizes)):
+            sizes[i] = val if i == self.side_index else 10
+        self.setSizes(sizes)
+        total = sum(self.sizes())
+        sizes = list(self.sizes())
+        for i in range(len(sizes)):
+            sizes[i] = val if i == self.side_index else total-val
+        self.setSizes(sizes)
+        self.initialize()
 
     def do_resize(self, *args):
         orig = self.desired_side_size
