@@ -1,3 +1,4 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
 '''
 Dynamic language lookup of translations for user-visible strings.
 '''
@@ -5,7 +6,7 @@ Dynamic language lookup of translations for user-visible strings.
 __license__   = 'GPL v3'
 __copyright__ = '2008, Marshall T. Vandegrift <llasram@gmail.com>'
 
-import cStringIO
+import io
 from gettext import GNUTranslations
 from calibre.utils.localization import get_lc_messages_path
 from zipfile import ZipFile
@@ -25,8 +26,8 @@ def translate(lang, text):
             with ZipFile(P('localization/locales.zip',
                 allow_user_override=False), 'r') as zf:
                 try:
-                    buf = cStringIO.StringIO(zf.read(mpath + '/messages.mo'))
-                except:
+                    buf = io.BytesIO(zf.read(mpath + '/messages.mo'))
+                except Exception:
                     pass
                 else:
                     trans = GNUTranslations(buf)
