@@ -6,12 +6,13 @@ Created on 13 Jan 2011
 
 @author: charles
 '''
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import inspect, re, traceback
+import inspect, re, traceback, numbers
 from math import trunc
 
 from calibre import human_readable
@@ -22,6 +23,7 @@ from calibre.utils.titlecase import titlecase
 from calibre.utils.icu import capitalize, strcmp, sort_key
 from calibre.utils.date import parse_date, format_date, now, UNDEFINED_DATE
 from calibre.utils.localization import calibre_langcode_to_name, canonicalize_lang
+from polyglot.builtins import iteritems, itervalues, unicode_type
 
 
 class FormatterFunctions(object):
@@ -62,7 +64,7 @@ class FormatterFunctions(object):
         self._register_functions()
 
     def _register_functions(self):
-        for compiled_funcs in self._functions_from_library.itervalues():
+        for compiled_funcs in itervalues(self._functions_from_library):
             for cls in compiled_funcs:
                 f = self._functions.get(cls.name, None)
                 replace = False
@@ -92,7 +94,7 @@ class FormatterFunctions(object):
 
     def get_builtins_and_aliases(self):
         res = {}
-        for f in self._builtins.itervalues():
+        for f in itervalues(self._builtins):
             res[f.name] = f
             for a in f.aliases:
                 res[a] = f
@@ -130,12 +132,12 @@ class FormatterFunction(object):
 
     def eval_(self, formatter, kwargs, mi, locals, *args):
         ret = self.evaluate(formatter, kwargs, mi, locals, *args)
-        if isinstance(ret, (str, unicode)):
+        if isinstance(ret, (bytes, unicode_type)):
             return ret
         if isinstance(ret, list):
             return ','.join(ret)
-        if isinstance(ret, (int, float, bool)):
-            return unicode(ret)
+        if isinstance(ret, (numbers.Number, bool)):
+            return unicode_type(ret)
 
 
 class BuiltinFormatterFunction(FormatterFunction):
@@ -245,7 +247,7 @@ class BuiltinAdd(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, x, y):
         x = float(x if x and x != 'None' else 0)
         y = float(y if y and y != 'None' else 0)
-        return unicode(x + y)
+        return unicode_type(x + y)
 
 
 class BuiltinSubtract(BuiltinFormatterFunction):
@@ -257,7 +259,7 @@ class BuiltinSubtract(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, x, y):
         x = float(x if x and x != 'None' else 0)
         y = float(y if y and y != 'None' else 0)
-        return unicode(x - y)
+        return unicode_type(x - y)
 
 
 class BuiltinMultiply(BuiltinFormatterFunction):
@@ -269,7 +271,7 @@ class BuiltinMultiply(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, x, y):
         x = float(x if x and x != 'None' else 0)
         y = float(y if y and y != 'None' else 0)
-        return unicode(x * y)
+        return unicode_type(x * y)
 
 
 class BuiltinDivide(BuiltinFormatterFunction):
@@ -281,7 +283,7 @@ class BuiltinDivide(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, x, y):
         x = float(x if x and x != 'None' else 0)
         y = float(y if y and y != 'None' else 0)
-        return unicode(x / y)
+        return unicode_type(x / y)
 
 
 class BuiltinTemplate(BuiltinFormatterFunction):
@@ -345,7 +347,7 @@ class BuiltinPrint(BuiltinFormatterFunction):
             'the output will go to a black hole.')
 
     def evaluate(self, formatter, kwargs, mi, locals, *args):
-        print args
+        print(args)
         return ''
 
 
@@ -373,7 +375,7 @@ class BuiltinRawField(BuiltinFormatterFunction):
             if fm is None:
                 return ', '.join(res)
             return fm['is_multiple']['list_to_ui'].join(res)
-        return unicode(res)
+        return unicode_type(res)
 
 
 class BuiltinRawList(BuiltinFormatterFunction):
@@ -525,43 +527,65 @@ class BuiltinStrcatMax(BuiltinFormatterFunction):
 
 class BuiltinInList(BuiltinFormatterFunction):
     name = 'in_list'
-    arg_count = 5
+    arg_count = -1
     category = 'List lookup'
-    __doc__ = doc = _('in_list(val, separator, pattern, found_val, not_found_val) -- '
+    __doc__ = doc = _('in_list(val, separator, pattern, found_val, ..., not_found_val) -- '
             'treat val as a list of items separated by separator, '
-            'comparing the pattern against each value in the list. If the '
+            'evaluating the pattern against each value in the list. If the '
             'pattern matches a value, return found_val, otherwise return '
-            'not_found_val.')
+            'not_found_val. The pattern and found_value can be repeated as '
+            'many times as desired, permitting returning different values '
+            'depending on the search. The patterns are checked in order. The '
+            'first match is returned.')
 
-    def evaluate(self, formatter, kwargs, mi, locals, val, sep, pat, fv, nfv):
+    def evaluate(self, formatter, kwargs, mi, locals, val, sep, *args):
+        if (len(args) % 2) != 1:
+            raise ValueError(_('in_list requires an odd number of arguments'))
         l = [v.strip() for v in val.split(sep) if v.strip()]
-        if l:
-            for v in l:
-                if re.search(pat, v, flags=re.I):
-                    return fv
-        return nfv
+        i = 0
+        while i < len(args):
+            if i + 1 >= len(args):
+                return args[i]
+            sf = args[i]
+            fv = args[i+1]
+            if l:
+                for v in l:
+                    if re.search(sf, v, flags=re.I):
+                        return fv
+            i += 2
 
 
 class BuiltinStrInList(BuiltinFormatterFunction):
     name = 'str_in_list'
-    arg_count = 5
+    arg_count = -1
     category = 'List lookup'
-    __doc__ = doc = _('str_in_list(val, separator, string, found_val, not_found_val) -- '
+    __doc__ = doc = _('str_in_list(val, separator, string, found_val, ..., not_found_val) -- '
             'treat val as a list of items separated by separator, '
             'comparing the string against each value in the list. If the '
-            'string matches a value, return found_val, otherwise return '
+            'string matches a value (ignoring case) then return found_val, otherwise return '
             'not_found_val. If the string contains separators, then it is '
-            'also treated as a list and each value is checked.')
+            'also treated as a list and each value is checked. The string and '
+            'found_value can be repeated as many times as desired, permitting '
+            'returning different values depending on the search. The strings are '
+            'checked in order. The first match is returned.')
 
-    def evaluate(self, formatter, kwargs, mi, locals, val, sep, str, fv, nfv):
+    def evaluate(self, formatter, kwargs, mi, locals, val, sep, *args):
+        if (len(args) % 2) != 1:
+            raise ValueError(_('str_in_list requires an odd number of arguments'))
         l = [v.strip() for v in val.split(sep) if v.strip()]
-        c = [v.strip() for v in str.split(sep) if v.strip()]
-        if l:
-            for v in l:
-                for t in c:
-                    if strcmp(t, v) == 0:
-                        return fv
-        return nfv
+        i = 0
+        while i < len(args):
+            if i + 1 >= len(args):
+                return args[i]
+            sf = args[i]
+            fv = args[i+1]
+            c = [v.strip() for v in sf.split(sep) if v.strip()]
+            if l:
+                for v in l:
+                    for t in c:
+                        if strcmp(t, v) == 0:
+                            return fv
+            i += 2
 
 
 class BuiltinIdentifierInList(BuiltinFormatterFunction):
@@ -617,7 +641,7 @@ class BuiltinReGroup(BuiltinFormatterFunction):
             'the template and the eval functions, you use [[ for { and ]] for }.'
             ' The following example in template program mode looks for series '
             'with more than one word and uppercases the first word: '
-            "{series:'re_group($, \"(\S* )(.*)\", \"[[$:uppercase()]]\", \"[[$]]\")'}")
+            "{series:'re_group($, \"(\\S* )(.*)\", \"[[$:uppercase()]]\", \"[[$]]\")'}")
 
     def evaluate(self, formatter, kwargs, mi, locals, val, pattern, *args):
         from formatter import EvalFormatter
@@ -703,7 +727,7 @@ class BuiltinCount(BuiltinFormatterFunction):
             'uses an ampersand. Examples: {tags:count(,)}, {authors:count(&)}')
 
     def evaluate(self, formatter, kwargs, mi, locals, val, sep):
-        return unicode(len([v for v in val.split(sep) if v]))
+        return unicode_type(len([v for v in val.split(sep) if v]))
 
 
 class BuiltinListitem(BuiltinFormatterFunction):
@@ -814,7 +838,7 @@ class BuiltinFormatsSizes(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals):
         fmt_data = mi.get('format_metadata', {})
         try:
-            return ','.join(k.upper()+':'+str(v['size']) for k,v in fmt_data.iteritems())
+            return ','.join(k.upper()+':'+unicode_type(v['size']) for k,v in iteritems(fmt_data))
         except:
             return ''
 
@@ -833,7 +857,7 @@ class BuiltinFormatsPaths(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals):
         fmt_data = mi.get('format_metadata', {})
         try:
-            return ','.join(k.upper()+':'+str(v['path']) for k,v in fmt_data.iteritems())
+            return ','.join(k.upper()+':'+unicode_type(v['path']) for k,v in iteritems(fmt_data))
         except:
             return ''
 
@@ -901,9 +925,9 @@ class BuiltinSublist(BuiltinFormatterFunction):
             'of zero is assumed to be the length of the list. Examples using '
             'basic template mode and assuming that the tags column (which is '
             'comma-separated) contains "A, B, C": '
-            '{tags:sublist(0,1,\,)} returns "A". '
-            '{tags:sublist(-1,0,\,)} returns "C". '
-            '{tags:sublist(0,-1,\,)} returns "A, B".'
+            '{tags:sublist(0,1,\\\\,)} returns "A". '
+            '{tags:sublist(-1,0,\\\\,)} returns "C". '
+            '{tags:sublist(0,-1,\\\\,)} returns "A, B".'
             )
 
     def evaluate(self, formatter, kwargs, mi, locals, val, start_index, end_index, sep):
@@ -1064,7 +1088,7 @@ class BuiltinBooksize(BuiltinFormatterFunction):
             try:
                 v = mi._proxy_metadata.book_size
                 if v is not None:
-                    return str(mi._proxy_metadata.book_size)
+                    return unicode_type(mi._proxy_metadata.book_size)
                 return ''
             except:
                 pass
@@ -1198,7 +1222,7 @@ class BuiltinListUnion(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
         res = [l.strip() for l in list1.split(separator) if l.strip()]
         l2 = [l.strip() for l in list2.split(separator) if l.strip()]
-        lcl1 = set([icu_lower(l) for l in res])
+        lcl1 = {icu_lower(l) for l in res}
 
         for i in l2:
             if icu_lower(i) not in lcl1 and i not in res:
@@ -1219,7 +1243,7 @@ class BuiltinListDifference(BuiltinFormatterFunction):
 
     def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
         l1 = [l.strip() for l in list1.split(separator) if l.strip()]
-        l2 = set([icu_lower(l.strip()) for l in list2.split(separator) if l.strip()])
+        l2 = {icu_lower(l.strip()) for l in list2.split(separator) if l.strip()}
 
         res = []
         for i in l1:
@@ -1241,7 +1265,7 @@ class BuiltinListIntersection(BuiltinFormatterFunction):
 
     def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
         l1 = [l.strip() for l in list1.split(separator) if l.strip()]
-        l2 = set([icu_lower(l.strip()) for l in list2.split(separator) if l.strip()])
+        l2 = {icu_lower(l.strip()) for l in list2.split(separator) if l.strip()}
 
         res = []
         for i in l1:
@@ -1280,8 +1304,8 @@ class BuiltinListEquals(BuiltinFormatterFunction):
             'The comparison is case insensitive.')
 
     def evaluate(self, formatter, kwargs, mi, locals, list1, sep1, list2, sep2, yes_val, no_val):
-        s1 = set([icu_lower(l.strip()) for l in list1.split(sep1) if l.strip()])
-        s2 = set([icu_lower(l.strip()) for l in list2.split(sep2) if l.strip()])
+        s1 = {icu_lower(l.strip()) for l in list1.split(sep1) if l.strip()}
+        s2 = {icu_lower(l.strip()) for l in list2.split(sep2) if l.strip()}
         if s1 == s2:
             return yes_val
         return no_val
@@ -1510,7 +1534,7 @@ class BuiltinUserCategories(BuiltinFormatterFunction):
 
     def evaluate(self, formatter, kwargs, mi, locals_):
         if hasattr(mi, '_proxy_metadata'):
-            cats = set(k for k, v in mi._proxy_metadata.user_categories.iteritems() if v)
+            cats = set(k for k, v in iteritems(mi._proxy_metadata.user_categories) if v)
             cats = sorted(cats, key=sort_key)
             return ', '.join(cats)
         return _('This function can be used only in the GUI')
@@ -1630,8 +1654,8 @@ class UserFunction(FormatterUserFunction):
 ''' + func
     locals_ = {}
     if DEBUG and tweaks.get('enable_template_debug_printing', False):
-        print prog
-    exec prog in locals_
+        print(prog)
+    exec(prog, locals_)
     cls = locals_['UserFunction'](name, doc, arg_count, eval_func)
     return cls
 
@@ -1661,7 +1685,7 @@ def load_user_template_functions(library_uuid, funcs, precompiled_user_functions
         compiled_funcs = precompiled_user_functions
     else:
         compiled_funcs = compile_user_template_functions(funcs)
-    formatter_functions().register_functions(library_uuid, compiled_funcs.values())
+    formatter_functions().register_functions(library_uuid, list(compiled_funcs.values()))
 
 
 def unload_user_template_functions(library_uuid):
