@@ -22,7 +22,7 @@ from calibre.utils.filenames import atomic_rename
 from calibre.utils.terminal import ANSIStream
 from duktape import Context, JSError, to_python
 from lzma.xz import compress, decompress
-from polyglot.builtins import itervalues, range, exec_path, raw_input, error_message, filter, getcwd, zip
+from polyglot.builtins import itervalues, range, exec_path, raw_input, error_message, filter, getcwd, zip, unicode_type
 from polyglot.queue import Empty, Queue
 
 COMPILER_PATH = 'rapydscript/compiler.js.xz'
@@ -39,7 +39,7 @@ def update_rapydscript():
     base = d(d(d(d(d(abspath(__file__))))))
     base = os.path.join(base, 'rapydscript')
     raw = subprocess.check_output(['node', '--harmony', os.path.join(base, 'bin', 'export')])
-    if isinstance(raw, type('')):
+    if isinstance(raw, unicode_type):
         raw = raw.encode('utf-8')
     path = P(COMPILER_PATH, allow_user_override=False)
     with open(path, 'wb') as f:
@@ -210,7 +210,7 @@ def compile_srv():
     rapydscript_dir = os.path.join(base, 'src', 'pyj')
     rb = os.path.join(base, 'src', 'calibre', 'srv', 'render_book.py')
     with lopen(rb, 'rb') as f:
-        rv = str(int(re.search(br'^RENDER_VERSION\s+=\s+(\d+)', f.read(), re.M).group(1)))
+        rv = unicode_type(int(re.search(br'^RENDER_VERSION\s+=\s+(\d+)', f.read(), re.M).group(1)))
     mathjax_version = json.loads(P('mathjax/manifest.json', data=True, allow_user_override=False))['etag']
     base = os.path.join(base, 'resources', 'content-server')
     fname = os.path.join(rapydscript_dir, 'srv.pyj')
@@ -273,7 +273,7 @@ def leading_whitespace(line):
 
 
 def format_error(data):
-    return ':'.join(map(type(''), (data['file'], data['line'], data['col'], data['message'])))
+    return ':'.join(map(unicode_type, (data['file'], data['line'], data['col'], data['message'])))
 
 
 class Repl(Thread):
